@@ -1,4 +1,4 @@
-/* $Id: composition_adjustment.h,v 1.15 2006/05/03 14:06:49 gertz Exp $
+/* $Id: composition_adjustment.h,v 1.18 2006/09/18 18:16:21 gertz Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -43,14 +43,17 @@ extern "C" {
 
 /* Some characters in the NCBIstdaa alphabet, including ambiguity
    characters, selenocysteine and the stop character. */
-enum { eGapChar = 0, eBchar = 2, eDchar = 4, eEchar = 5, eNchar = 13,
-       eQchar = 15, eXchar = 21, eZchar = 23, eSelenocysteine = 24,
-       eStopChar = 25};
+enum { eGapChar = 0, eBchar = 2,  eDchar = 4,  eEchar = 5, eIchar = 9,
+       eLchar = 11,  eNchar = 13, eQchar = 15, eXchar = 21,
+       eZchar = 23,  eSelenocysteine = 24, eStopChar = 25,
+       eOchar = 26,  eJchar = 27};
 
 /**
- * Represents the composition of an amino-acid sequence */
+ * Represents the composition of an amino-acid sequence, in the ncbistdaa
+ * alphabet. */
 typedef struct Blast_AminoAcidComposition {
-    double prob[COMPO_PROTEIN_ALPHABET];   /**< probabilities of each amino acid */
+    /** probabilities of each amino acid */
+    double prob[COMPO_LARGEST_ALPHABET];
     int numTrueAminoAcids;   /**< number of true amino acids in the sequence,
                                   omitting nonstandard amino acids */
 } Blast_AminoAcidComposition;
@@ -61,11 +64,13 @@ typedef struct Blast_AminoAcidComposition {
  * ambiguity characters and other nonstandard characters.
  *
  * @param composition      the computed composition
+ * @param alphsize         the size of the alphabet
  * @param sequence         a sequence of amino acids
  * @param length           length of the sequence
  */
 NCBI_XBLAST_EXPORT
 void Blast_ReadAaComposition(Blast_AminoAcidComposition * composition,
+                             int alphsize,
                              const Uint1 * sequence, int length);
 
 
@@ -77,6 +82,8 @@ typedef struct Blast_MatrixInfo {
                                     scores */
     int      rows;             /**< the number of rows in the scoring
                                     matrix. */
+    int      cols;             /**< the number of columns in the scoring
+                                    matrix, i.e. the alphabet size. */
     int      positionBased;    /**< is the matrix position-based */
     double   ungappedLambda;   /**< ungapped Lambda value for this matrix
                                     in standard context */
@@ -85,13 +92,15 @@ typedef struct Blast_MatrixInfo {
 
 /** Create a Blast_MatrixInfo object
  *
- *  @param rows        the number of rows in the matrix, should be
- *                     COMPO_PROTEIN_ALPHABET unless the matrix is position
- *                     based, in which case it is the query length
+ *  @param rows        the number of rows in the matrix, should be equal
+ *                     to the size of the alphabet unless the matrix is
+ *                     position based, in which case it is the query length
+ *  @param cols        the number of columns in the matrix; the size of the
+ *                     alphabet
  *  @param positionBased  is this matrix position-based?
  */
 NCBI_XBLAST_EXPORT
-Blast_MatrixInfo * Blast_MatrixInfoNew(int rows, int positionBased);
+Blast_MatrixInfo * Blast_MatrixInfoNew(int rows, int cols, int positionBased);
 
 
 /** Free memory associated with a Blast_MatrixInfo object */
@@ -202,6 +211,7 @@ Blast_CompositionBasedStats(int ** matrix, double * LambdaRatio,
  * to optimize a score matrix to a given set of letter frequencies.
  *
  * @param matrix       the newly computed matrix [out]
+ * @param alphsize     the size of the alphabet [in]
  * @param matrix_adjust_rule    the rule to use when computing the matrix;
  *                              affects how the relative entropy is
  *                              constrained
@@ -227,6 +237,7 @@ Blast_CompositionBasedStats(int ** matrix, double * LambdaRatio,
  */
 NCBI_XBLAST_EXPORT
 int Blast_CompositionMatrixAdj(int ** matrix,
+                               int alphsize,
                                EMatrixAdjustRule matrix_adjust_rule,
                                int length1, int length2,
                                const double *probArray1,
@@ -287,11 +298,13 @@ Blast_AdjustScores(Int4 ** matrix,
  * score frequencies.
  *
  * @param matrix       the preallocated matrix
+ * @param size         size of the matrix
  * @param freq         a set of score frequencies
  * @param Lambda       the desired scale of the matrix
  */
 NCBI_XBLAST_EXPORT
-void Blast_Int4MatrixFromFreq(Int4 **matrix, double ** freq, double Lambda);
+void Blast_Int4MatrixFromFreq(Int4 **matrix, int size, 
+                              double ** freq, double Lambda);
 
 
 /**
@@ -341,6 +354,22 @@ int Blast_EntropyOldFreqNewContext(double * entropy, double * Lambda,
                                    const double row_prob[],
                                    const double col_prob[]);
 
+/**
+ * Convert a matrix of target frequencies for the ARND alphabet of
+ * true amino acids to a set of target frequencies for the NCBIstdaa
+ * alphabet, filling in value for the two-character ambiguities (but
+ * not X).
+ *
+ * @param StdFreq      frequencies in the NCBIstdaa alphabet [output]
+ * @param StdAlphsize  the size of the NCBIstdaa alphabet [input]
+ * @param freq         frequencies in the ARND alphabet [input]
+ */
+NCBI_XBLAST_EXPORT
+void
+Blast_TrueAaToStdTargetFreqs(double ** StdFreq, int StdAlphsize,
+                             double ** freq);
+
+
 /**
  * Convert a matrix of frequency ratios to a matrix of scores.
  * @param matrix            the matrix
@@ -377,22 +406,22 @@ void Blast_CalcFreqRatios(double ** ratios, int alphsize,
 /**
  * Find the weighted average of a set of observed probabilities with a
  * set of "background" probabilities.  All array parameters have
- * length COMPO_NUM_TRUE_AA.
+ * length COMPO_NUM_TRUE_AA (i.e. 20).
  *
- * @param probs                   on entry, observed probabilities; on
+ * @param probs20                 on entry, observed probabilities; on
  *                                exit, weighted average probabilities.
  * @param number_of_observations  the number of characters used to
  *                                form the observed_freq array
- * @param background_probs        the probability of characters in a
+ * @param background_probs20      the probability of characters in a
  *                                standard sequence.
  * @param pseudocounts            the number of "standard" characters
  *                                to be added to form the weighted
  *                                average.
  */
 NCBI_XBLAST_EXPORT
-void Blast_ApplyPseudocounts(double * probs,
+void Blast_ApplyPseudocounts(double * probs20,
                              int number_of_observations,
-                             const double * background_probs,
+                             const double * background_probs20,
                              int pseudocounts);
 
 /**
