@@ -1,4 +1,4 @@
-/* $Id: blast_filter.h,v 1.28 2005/03/02 13:52:59 madden Exp $
+/* $Id: blast_filter.h,v 1.39 2007/07/25 12:55:50 kazimird Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -32,10 +32,13 @@
  * functions, combine with blast_dust.h?
  */
 
-#ifndef __BLAST_FILTER__
-#define __BLAST_FILTER__
+#ifndef ALGO_BLAST_CORE__BLAST_FILTER__H
+#define ALGO_BLAST_CORE__BLAST_FILTER__H
 
+#include <algo/blast/core/ncbi_std.h>
 #include <algo/blast/core/blast_def.h>
+#include <algo/blast/core/blast_program.h>
+#include <algo/blast/core/blast_query_info.h>
 #include <algo/blast/core/blast_message.h>
 #include <algo/blast/core/blast_options.h>
 
@@ -51,6 +54,7 @@ extern const Uint1 kProtMask;
 /** Repeats filtering default options. */
 #define REPEATS_SEARCH_EVALUE 0.1       /**< Default e-value threshold */
 #define REPEATS_SEARCH_PENALTY -1       /**< Default mismatch penalty */
+#define REPEATS_SEARCH_REWARD 1       /**< Default match reward */
 #define REPEATS_SEARCH_GAP_OPEN 2       /**< Default gap opening cost */
 #define REPEATS_SEARCH_GAP_EXTEND 1     /**< Default gap extension cost */
 #define REPEATS_SEARCH_WORD_SIZE 11     /**< Default word size */
@@ -65,15 +69,32 @@ extern const Uint1 kProtMask;
 #define REPEAT_MASK_LINK_VALUE 5
 
 /** Create and initialize a new sequence interval.
- * @param head existing BlastSeqLoc to add onto, if *head
+ * @param head existing BlastSeqLoc to append onto, if *head
  *   is NULL then it will be set to new BlastSeqLoc, may be NULL [in|out]
  * @param from Start of the interval [in]
  * @param to End of the interval [in]
- * @return Pointer to the allocated BlastSeqLoc structure.
+ * @return Pointer to the allocated BlastSeqLoc structure (i.e.: tail of the
+ * list).
  */
 NCBI_XBLAST_EXPORT
 BlastSeqLoc* BlastSeqLocNew(BlastSeqLoc** head, Int4 from, Int4 to);
 
+/** Appends the BlastSeqLoc to the list of BlastSeqLoc-s pointed to by head.
+ * @param head Pointer to the head of the linked list of BlastSeqLoc-s [in]
+ * @param node Pointer to the node to be added to the list. If this is NULL,
+ * this function does nothing. [in]
+ * @returns pointer to the second argument to this function (i.e.: tail of the
+ * list)
+ */
+BlastSeqLoc* BlastSeqLocAppend(BlastSeqLoc** head, BlastSeqLoc* node);
+
+/** Deallocate a single BlastSeqLoc structure and its contents, without
+ * following its next pointer
+ * @param node structure to deallocate [in]
+ * @return NULL
+ */
+BlastSeqLoc* BlastSeqLocNodeFree(BlastSeqLoc* node);
+
 /** Deallocate all BlastSeqLoc objects in a chain.
  * @param loc object to be freed [in]
  * @return NULL pointer returned.
@@ -81,6 +102,56 @@ BlastSeqLoc* BlastSeqLocNew(BlastSeqLoc** head, Int4 from, Int4 to);
 NCBI_XBLAST_EXPORT
 BlastSeqLoc* BlastSeqLocFree(BlastSeqLoc* loc);
 
+/** Make a deep copy of the linked list of BlastSeqLoc-s pointed to by its
+ * argument
+ * @param head head of the linked list [in]
+ * @return NULL on NULL input or memory allocation failure, else a copy of the
+ * list and its contents
+ */
+BlastSeqLoc* BlastSeqLocListDup(BlastSeqLoc* head);
+
+/** Converts reverse strand coordinates to forward strand.
+ * @param filter_in BlastSeqLoc to be reversed [in]
+ * @param query_length length of query [in]
+ * @return reversed BlastSeqLoc
+ */
+NCBI_XBLAST_EXPORT
+BlastSeqLoc* BlastSeqLocReverse(const BlastSeqLoc* filter_in, 
+                                Int4 query_length);
+
+/** Go through all mask locations in one sequence, 
+ * combine any that overlap. Deallocate the memory for the locations that 
+ * were on the list, produce a new (merged) list of locations. 
+ * @param mask_loc The list of masks to be merged [in] 
+ * @param link_value Largest gap size between locations for which they
+ *                   should be linked together [in] 
+ * @return The new (merged) list of masks or NULL if mask_loc is NULL or memory
+ * allocation failure.
+*/
+NCBI_XBLAST_EXPORT
+BlastSeqLoc*
+BlastSeqLocCombine(BlastSeqLoc* mask_loc, Int4 link_value);
+
+/** Allocate memory for a BlastMaskLoc.
+ * @param total number of contexts for which SSeqLocs should be allocated 
+ * (result of number of queries * number of contexts for given program) [in]
+ * @return Pointer to the allocated BlastMaskLoc structure.
+*/
+NCBI_XBLAST_EXPORT
+BlastMaskLoc* BlastMaskLocNew(Int4 total);
+
+/** 
+ * @brief Perform a deep copy of the BlastMaskLoc structure passed to this
+ * function
+ * 
+ * @param mask_loc Source masking location structure [in]
+ * 
+ * @return Deep copy of its argument, or NULL if the argument was NULL or if
+ * not enough memory was available
+ */
+NCBI_XBLAST_EXPORT
+BlastMaskLoc* BlastMaskLocDup(const BlastMaskLoc* mask_loc);
+
 /** Deallocate memory for a BlastMaskLoc structure
  * as well as the BlastSeqLoc's pointed to.
  * @param mask_loc the object to be deleted [in]
@@ -89,18 +160,17 @@ BlastSeqLoc* BlastSeqLocFree(BlastSeqLoc* loc);
 NCBI_XBLAST_EXPORT
 BlastMaskLoc* BlastMaskLocFree(BlastMaskLoc* mask_loc);
 
-/** Allocate memory for a BlastMaskLoc, also allocates array for BlastSeqLoc* of length total.
- * @param total which context (i.e., strand) [in]
- * @return Pointer to the allocated BlastMaskLoc structure.
-*/
-NCBI_XBLAST_EXPORT
-BlastMaskLoc* BlastMaskLocNew(Int4 total);
-
 /** Given a BlastMaskLoc with an array of lists of DNA mask locations, 
  * substitutes that array by a new array of per-protein-frame mask location 
  * lists.
- * @param mask_loc Mask locations structure [in|out]
+ * @param mask_loc Mask locations structure. This structure can have either
+ * masks for all frames in nucleotide coordinates (e.g.: the results of
+ * translating protein masks to nucleotide) or a single mask per query
+ * (i.e.:location NUM_FRAMES*query_index). In the latter case, this mask will 
+ * be used for all frames. [in|out]
  * @param query_info Query information structure, containing contexts data [in]
+ * @note This function does NOT take into consideration the strands requested
+ * to be searched, which is INCONSISTENT with what the C++ API does.
  */
 Int2 BlastMaskLocDNAToProtein(BlastMaskLoc* mask_loc, 
                               const BlastQueryInfo* query_info);
@@ -113,19 +183,6 @@ Int2 BlastMaskLocDNAToProtein(BlastMaskLoc* mask_loc,
 Int2 BlastMaskLocProteinToDNA(BlastMaskLoc* mask_loc, 
                               const BlastQueryInfo* query_info);
 
-/** Go through all mask locations in one sequence, 
- * combine any that overlap. Deallocate the memory for the locations that 
- * were on the list, produce a new (merged) list of locations. 
- * @param mask_loc The list of masks to be merged [in] 
- * @param mask_loc_out The new (merged) list of masks. [out]
- * @param link_value Largest gap size between locations fow which they
- *                   should be linked together [in] 
-*/
-NCBI_XBLAST_EXPORT
-Int2
-CombineMaskLocations(BlastSeqLoc* mask_loc, BlastSeqLoc* *mask_loc_out,
-                     Int4 link_value);
-
 /** This function takes the list of mask locations (i.e., regions that 
  * should not be searched or not added to lookup table) and makes up a set 
  * of SSeqRange*'s in the concatenated sequence built from a set of queries,
@@ -141,14 +198,13 @@ CombineMaskLocations(BlastSeqLoc* mask_loc, BlastSeqLoc* *mask_loc_out,
  * @param complement_mask Linked list of SSeqRange*s in the concatenated 
  *                        sequence to be indexed in the lookup table . [out]
  */
-NCBI_XBLAST_EXPORT
 Int2 
 BLAST_ComplementMaskLocations(EBlastProgramType program_number, 
-   BlastQueryInfo* query_info, BlastMaskLoc* mask_loc, 
+   const BlastQueryInfo* query_info, const BlastMaskLoc* mask_loc, 
    BlastSeqLoc* *complement_mask);
 
-/** Runs filtering functions, according to the filtering options, returns
- * SeqLocPtr. Should combine all SeqLocs so they are non-redundant.
+/** Runs seg filtering functions, according to the filtering options, returns
+ * BlastSeqLoc*. Should combine all SeqLocs so they are non-redundant.
  * @param program_number Type of BLAST program [in]
  * @param sequence The sequence or part of the sequence to be filtered [in]
  * @param length Length of the (sub)sequence [in]
@@ -179,25 +235,42 @@ BlastSetUp_Filter(EBlastProgramType program_number,
 */
 NCBI_XBLAST_EXPORT
 Int2
-BlastSetUp_GetFilteringLocations(BLAST_SequenceBlk* query_blk, BlastQueryInfo* query_info,
-    EBlastProgramType program_number, const SBlastFilterOptions* filter_options, 
-    BlastMaskLoc** filter_out, Blast_Message* *blast_message);
+BlastSetUp_GetFilteringLocations(BLAST_SequenceBlk* query_blk, 
+                                 const BlastQueryInfo* query_info, 
+                                 EBlastProgramType program_number, 
+                                 const SBlastFilterOptions* filter_options, 
+                                 BlastMaskLoc** filter_out, 
+                                 Blast_Message* *blast_message);
 
 /** Masks the letters in buffer.
  * This is a low-level routine and takes a raw buffer which it assumes
  * to be in ncbistdaa (protein) or blastna (nucleotide).
- * @param buffer the sequence to be masked (will be modified). [out]
+ * @param buffer the sequence to be masked (will be modified, cannot be NULL or
+ * undefined behavior will result).[in|out]
  * @param length length of the sequence to be masked . [in]
  * @param is_na nucleotide if TRUE [in]
  * @param mask_loc the BlastSeqLoc to use for masking [in] 
  * @param reverse minus strand if TRUE [in]
  * @param offset how far along sequence is 1st residuse in buffer [in]
- *
 */
 NCBI_XBLAST_EXPORT
-Int2
+void
 Blast_MaskTheResidues(Uint1 * buffer, Int4 length, Boolean is_na, 
-    BlastSeqLoc* mask_loc, Boolean reverse, Int4 offset);
+    const BlastSeqLoc* mask_loc, Boolean reverse, Int4 offset);
+
+/** Mask protein letters that are currently unsupported. This routine
+ *  is used to make the core ignore letters within protein sequences
+ *  that cannot (yet) be correctly handled
+ * @param seq Protein sequence to be masked (ncbistdaa format required).
+ *            Letters whose numerical value exceeds a cutoff are
+ *            converted into kProtMask values [in|out]
+ * @param min_invalid The first ncbistdaa value that is considered invalid.
+ *            All sequence letters with numerical value >= this number
+ *            are masked [in]
+ */
+NCBI_XBLAST_EXPORT
+void
+Blast_MaskUnsupportedAA(BLAST_SequenceBlk* seq, Uint1 min_invalid);
 
 /** Masks the sequence given a BlastMaskLoc
  * @param query_blk sequence to be filtered [in]
@@ -206,9 +279,11 @@ Blast_MaskTheResidues(Uint1 * buffer, Int4 length, Boolean is_na,
  * @param program_number one of blastn,blastp,blastx,etc. [in]
 */
 NCBI_XBLAST_EXPORT
-Int2
-BlastSetUp_MaskQuery(BLAST_SequenceBlk* query_blk, BlastQueryInfo* query_info,
-    BlastMaskLoc *filter_maskloc, EBlastProgramType program_number);
+void
+BlastSetUp_MaskQuery(BLAST_SequenceBlk* query_blk, 
+                     const BlastQueryInfo* query_info, 
+                     const BlastMaskLoc *filter_maskloc, 
+                     EBlastProgramType program_number);
 
 /** Produces SBlastFilterOptions from a string that has been traditionally supported
  * in blast.
@@ -220,10 +295,24 @@ BlastSetUp_MaskQuery(BLAST_SequenceBlk* query_blk, BlastQueryInfo* query_info,
  */
 NCBI_XBLAST_EXPORT
 Int2
-BlastFilteringOptionsFromString(EBlastProgramType program_number, const char* instructions, 
-    SBlastFilterOptions* *filtering_options, Blast_Message* *blast_message);
+BlastFilteringOptionsFromString(EBlastProgramType program_number, 
+                                const char* instructions, 
+                                SBlastFilterOptions* *filtering_options, 
+                                Blast_Message* *blast_message);
+
+/** Determines whether this is a nucleotide query and whether this a minus strand or not
+ *
+ * @param is_na the query is nucleotide
+ * @param context offset in the QueryInfo array
+ * @return TRUE if this is minus strand
+ */
+static NCBI_INLINE Boolean BlastIsReverseStrand(Boolean is_na, Int4 context)
+{
+     return (is_na && ((context & 1) != 0));
+
+}
 
 #ifdef __cplusplus
 }
 #endif
-#endif /* !__BLAST_FILTER__ */
+#endif /* !ALGO_BLAST_CORE__BLAST_FILTER__H */
