@@ -114,7 +114,7 @@ AliConfigInfoPtr Ali_GetConfig (void)
       s_configInfo.declaredInfo.seqLength     = 0;
 
       s_configInfo.gapChar = (CharPtr) MemNew (32);
-      sprintf (s_configInfo.gapChar    , "%c", ALI_GAP_CHAR);
+      sprintf (s_configInfo.gapChar    , "%c%c", ALI_GAP_CHAR, '.');
       s_configInfo.missingChar = (CharPtr) MemNew (32);
       sprintf (s_configInfo.missingChar, "%c", ALI_MISSING_CHAR);
 
@@ -264,7 +264,7 @@ Boolean Ali_SetConfig (AliConfigInfoPtr configPtr,
       s_configInfo.declaredInfo.idCount       = 0;
       s_configInfo.declaredInfo.seqLength     = 0;
       s_configInfo.gapChar = (CharPtr) MemNew (32);
-      sprintf (s_configInfo.gapChar    , "%c", ALI_GAP_CHAR);
+      sprintf (s_configInfo.gapChar    , "%c%c", ALI_GAP_CHAR, '.');
       s_configInfo.missingChar = (CharPtr) MemNew (32);
       sprintf (s_configInfo.missingChar, "%c", ALI_MISSING_CHAR);
     }
@@ -545,6 +545,9 @@ static CharPtr s_GetRowIdString (ValNodePtr row)
   DefLineInfoPtr   defLinePtr;
   OtherLineInfoPtr otherLinePtr;
 
+  if (row == NULL)
+    return NULL;
+
   if (row->choice == ALI_SEQLINE)
     {
       seqLinePtr = (SeqLineInfoPtr) row->data.ptrvalue;
@@ -589,6 +592,9 @@ static CharPtr s_GetRowSeqString (ValNodePtr row)
   CharPtr          newSeqStr;
   SeqLineInfoPtr   seqLinePtr;
 
+  if (row == NULL)
+    return NULL;
+
   if (row->choice == ALI_SEQLINE)
     {
       seqLinePtr = (SeqLineInfoPtr) row->data.ptrvalue;
@@ -1825,6 +1831,7 @@ ErrInfoPtr Ali_AddError (ErrInfoPtr PNTR errorListPtr,
   Int4       declaredLen;
   Int4       sequenceCount;
   Int4       errorCount;
+  Int4       invalidChar;
 
   static Int4 count = 0;
 
@@ -1932,9 +1939,10 @@ ErrInfoPtr Ali_AddError (ErrInfoPtr PNTR errorListPtr,
       break;
     case ERR_INVALID_DEFLINE :
       defLineStr = va_arg (argPtr, CharPtr);
+      invalidChar = va_arg (argPtr, Int4);
       newError->info = (CharPtr) MemNew (strlen (defLineStr) + 100);
-      sprintf (newError->info, "Invalid definitions line :\n%s",
-	       defLineStr);
+      sprintf (newError->info, "Invalid definitions line (illegal char '%c'):\n%s",
+	       (Char) invalidChar, defLineStr);
       break;
     case ERR_DEFLINE_NODEFS :
       defLineStr = va_arg (argPtr, CharPtr);
@@ -2354,6 +2362,39 @@ static void s_SortErrors (AlignFileDataPtr fileInfoPtr)
   return;
 }
 
+/*=========================================================================*/
+/*                                                                         */
+/* s_ReplaceUWithT () -- Replace all the Us in a nucleotide sequence with  */
+/*                       Ns.                                               */
+/*                                                                         */
+/*=========================================================================*/
+
+static void s_ReplaceUWithT (AlignFileDataPtr fileInfoPtr)
+{
+  IdInfoPtr  seqPtr = NULL;
+  SeqPartPtr seqPart = NULL;
+  CharPtr    seqString;
+  Int4       i;
+
+  seqPtr = fileInfoPtr->sequences;
+  while (seqPtr != NULL)
+    {
+      seqPart = seqPtr->sequence;
+      while (seqPart != NULL)
+	{
+	  seqString = seqPart->sequence;
+	  for (i = 0; seqString[i] != '\0'; i++)
+	    if (seqString[i] == 'U')
+	      seqString[i] = 'T';
+	    else if (seqString[i] == 'u')
+	      seqString[i] = 't';
+	  seqPart = seqPart->next;
+	}
+      seqPtr = seqPtr->next;
+    }
+
+}
+
 /*=========================================================================*/
 /*                                                                         */
 /* s_AnalyzeErrors () -- Look for patterns in the errors that can be used  */
@@ -2541,15 +2582,20 @@ AlignFileDataPtr Ali_Read (FILE PNTR alignFilePtr)
 
   /* Return the missing, gap, and unaligned chars used */
 
-  fileInfoPtr->info->missingChar = (CharPtr) MemNew(2);
+  fileInfoPtr->info->missingChar = (CharPtr) MemNew(16);
   StringCpy (fileInfoPtr->info->missingChar, s_configInfo.missingChar);
 
-  fileInfoPtr->info->gapChar = (CharPtr) MemNew(2);
+  fileInfoPtr->info->gapChar = (CharPtr) MemNew(16);
   StringCpy (fileInfoPtr->info->gapChar, s_configInfo.gapChar);
 
-  fileInfoPtr->info->unalignedChar = (CharPtr) MemNew(2);
+  fileInfoPtr->info->unalignedChar = (CharPtr) MemNew(16);
   StringCpy (fileInfoPtr->info->unalignedChar, s_configInfo.unalignedChar);
 
+  /* If these are nucleotide sequences, then */
+  /* replace all 'U's with 'T's.             */
+
+  s_ReplaceUWithT (fileInfoPtr);
+
   /* Clean up and return successfully */
   
   s_FreeRowList_Safe (rowList);
@@ -2853,6 +2899,8 @@ static SeqAlignPtr ALI_MakeSeqAlign(AlignFileDataPtr afp, CharPtr PNTR PNTR stri
          buf[ctr] = '\0';
       }
       sip = MakeSeqID(iip->id);
+      SeqIdSetFree(sip->next);
+      sip->next = NULL;
       deflines[i] = StringSave(iip->defline);
       if (sip_prev != NULL)
       {
@@ -3205,22 +3253,24 @@ NLM_EXTERN SeqEntryPtr ALI_ConvertToNCBIData(AlignFileDataPtr afp)
    {
       len = StringLen(strings[i]);
       sep = StringToSeqEntry (strings[i], sip, len, moltype);
-      bsp = (BioseqPtr)(sep->data.ptrvalue);
-      if (! StringHasNoText (deflines[i])) {
-        str = deflines[i];
-        sdp = SeqDescrAddPointer(&(bsp->descr), Seq_descr_title, str);
+      if (sep != NULL) {
+	bsp = (BioseqPtr)(sep->data.ptrvalue);
+	if (! StringHasNoText (deflines[i])) {
+	  str = deflines[i];
+	  sdp = SeqDescrAddPointer(&(bsp->descr), Seq_descr_title, str);
+	}
+	if (sep != NULL)
+	  {
+	    if (sep_head != NULL)
+	      {
+		sep_prev->next = sep;
+		sep_prev = sep;
+	      } else
+		sep_head = sep_prev = sep;
+	  }
+	sip = sip->next;
+	MemFree(strings[i]);
       }
-      if (sep != NULL)
-      {
-         if (sep_head != NULL)
-         {
-            sep_prev->next = sep;
-            sep_prev = sep;
-         } else
-            sep_head = sep_prev = sep;
-      }
-      sip = sip->next;
-      MemFree(strings[i]);
    }
    sep_head = ALI_make_seqentry_for_seqentry (sep_head);
    SeqAlignAddInSeqEntry (sep_head, sanp);
