@@ -28,6 +28,17 @@
 * Author:  Karl Sirotkin
 *
 * $Log: gbparint.c,v $
+* Revision 6.8  2004/07/22 16:08:35  bazhin
+* Changes to parse gaps of unknown lengths (like "gap(unk100)")
+* within location strings.
+*
+* Revision 6.7  2004/03/03 17:32:19  kans
+* Nlm_gbparselex checks against NULL input
+*
+* Revision 6.6  2003/12/05 16:42:11  bazhin
+* Nlm_gbparselex() and Nlm_gbparselex_ver() functions now can handle
+* RefSeq and WGS accessions.
+*
 * Revision 6.5  2001/06/07 17:00:54  tatiana
 * added gi option in Nlm_gbparselex()
 *
@@ -91,7 +102,8 @@
 #define TAKE_FIRST 1
 #define TAKE_SECOND 2
 
-void Nlm_gbgap PROTO((ValNodePtr PNTR currentPt, ValNodePtr PNTR retval));
+void Nlm_gbgap PROTO((ValNodePtr PNTR currentPt, ValNodePtr PNTR retval,
+                      Boolean unknown));
 
 /*--------- do_Nlm_gbparse_error () ---------------*/
 
@@ -683,7 +695,10 @@ Nlm_gbloc(Boolean PNTR keep_rawPt, int PNTR parenPt, Boolean PNTR sitesPt, ValNo
 
 /* Interval, occurs on recursion */
 			case GBPARSE_INT_GAP:
-				Nlm_gbgap(currentPt, &retval);
+				Nlm_gbgap(currentPt, &retval, FALSE);
+				break;
+			case GBPARSE_INT_UNK_GAP:
+				Nlm_gbgap(currentPt, &retval, TRUE);
 				break;
 			case  GBPARSE_INT_ACCESION :
 			case  GBPARSE_INT_CARET : case  GBPARSE_INT_GT :
@@ -914,7 +929,10 @@ NLM_EXTERN SeqLocPtr Nlm_gbloc_ver(Boolean PNTR keep_rawPt, int PNTR parenPt,
 
 /* Interval, occurs on recursion */
 			case GBPARSE_INT_GAP:
-				Nlm_gbgap(currentPt, &retval);
+				Nlm_gbgap(currentPt, &retval, FALSE);
+				break;
+			case GBPARSE_INT_UNK_GAP:
+				Nlm_gbgap(currentPt, &retval, TRUE);
 				break;
 			case  GBPARSE_INT_ACCESION :
 			case  GBPARSE_INT_CARET : case  GBPARSE_INT_GT :
@@ -1056,7 +1074,8 @@ FATAL:
 }
 
 /**********************************************************/
-void Nlm_gbgap(ValNodePtr PNTR currentPt, ValNodePtr PNTR retval)
+void Nlm_gbgap(ValNodePtr PNTR currentPt, ValNodePtr PNTR retval,
+               Boolean unknown)
 {
     ValNodePtr vnp_first;
     ValNodePtr vnp_second;
@@ -1083,7 +1102,7 @@ void Nlm_gbgap(ValNodePtr PNTR currentPt, ValNodePtr PNTR retval)
         if(vnp_third == NULL || vnp_third->choice != GBPARSE_INT_RIGHT)
             return;
 
-        vvv = GapToSeqLoc(atoi((CharPtr) vnp_second->data.ptrvalue));
+        vvv = GapToSeqLocEx(atoi((CharPtr) vnp_second->data.ptrvalue), unknown);
         if(vvv == NULL)
             return;
 
@@ -1830,6 +1849,35 @@ NLM_EXTERN SeqLocPtr Nlm_gbreplace_ver(Boolean PNTR keep_rawPt, int PNTR parenPt
 	return retval;
 }
 
+/**********************************************************/
+static int Nlm_gbparse_accprefix(CharPtr acc)
+{
+    CharPtr p;
+    int     ret;
+
+    if(acc == NULL || *acc == '\0')
+        return(0);
+
+    for(p = acc; IS_ALPHA(*p) != 0;)
+        p++;
+    ret = p - acc;
+    if(*p == '_')
+    {
+        if(ret == 2)
+        {
+            for(p++; IS_ALPHA(*p) != 0;)
+                p++;
+            ret = p - acc;
+            if(ret != 3 && ret != 7)
+                ret = 1;
+        }
+        else
+            ret = 1;
+    }
+    else if(ret != 1 && ret != 2 && ret != 4)
+        ret = 1;
+    return(ret);
+}
 
 
 char Saved_ch;
@@ -1858,9 +1906,12 @@ Nlm_gbparselex(CharPtr linein, ValNodePtr PNTR lexed)
 	Boolean die_now=FALSE;
 	ValNode forerrmacro;
 
+
+	if (linein == NULL) return retval;
+
 	forerrmacro.choice =GBPARSE_INT_ACCESION ;
 
-	if (*linein	){
+	if (*linein){
 		len = StringLen(linein);
 		line_use = MemNew(len + 1);
 		StringCpy(line_use, linein);
@@ -1919,7 +1970,7 @@ Nlm_gbparselex(CharPtr linein, ValNodePtr PNTR lexed)
 			case '5': case '6': case '7': case '8': case '9':
 				skip_new_token = FALSE;
 				current_token -> choice = GBPARSE_INT_NUMBER;
-				for (dex=0, spare = current_col; isdigit(*spare); spare ++){
+				for (dex=0, spare = current_col; isdigit((int) *spare); spare ++){
 					dex ++ ;
 				}
 				current_token -> data.ptrvalue = MemNew(dex+1);
@@ -1994,9 +2045,17 @@ Nlm_gbparselex(CharPtr linein, ValNodePtr PNTR lexed)
 				    current_col[3] == '\0'))
 				{
 				    current_token->choice = GBPARSE_INT_GAP;
-				    current_col += 2;
 				    current_token->data.ptrvalue = MemNew(4);
 				    StringCpy(current_token->data.ptrvalue, "gap");
+				    if(StringNICmp(current_col + 3, "(unk", 4) == 0)
+				    {
+					current_token->choice = GBPARSE_INT_UNK_GAP;
+					last_token = current_token;
+					current_token = ValNodeNew(current_token);
+					current_token->choice = GBPARSE_INT_LEFT;
+					current_col += 4;
+				    }
+				    current_col += 2;
 				    break;
 				}
 				if(StringNCmp(current_col, "gi|", 3) == 0) {
@@ -2176,19 +2235,14 @@ Nlm_gbparselex(CharPtr linein, ValNodePtr PNTR lexed)
 /* new accessions have .version !!  2/15/1999 */
 				skip_new_token = FALSE;
 				current_token -> choice = GBPARSE_INT_ACCESION;
-				if (IS_ALPHA(*(current_col + 1))) {
-					spare = current_col + 2;
-					dex = 2;
-				} else {
-					spare = current_col + 1;
-					dex = 1;
-				}
-				for (; isdigit(*spare); spare ++){
+				dex = Nlm_gbparse_accprefix(current_col);
+				spare = current_col + dex;
+				for (; isdigit((int) *spare); spare ++){
 					dex ++ ;
 				}
 				if (*spare == '.') {
 					dex ++ ;
-					for (spare++; isdigit(*spare); spare ++){
+					for (spare++; isdigit((int) *spare); spare ++){
 						dex ++ ;
 					}
 				}
@@ -2302,7 +2356,7 @@ Nlm_gbparselex_ver(CharPtr linein, ValNodePtr PNTR lexed, Boolean accver)
 			case '5': case '6': case '7': case '8': case '9':
 				skip_new_token = FALSE;
 				current_token -> choice = GBPARSE_INT_NUMBER;
-				for (dex=0, spare = current_col; isdigit(*spare); spare ++){
+				for (dex=0, spare = current_col; isdigit((int) *spare); spare ++){
 					dex ++ ;
 				}
 				current_token -> data.ptrvalue = MemNew(dex+1);
@@ -2377,9 +2431,17 @@ Nlm_gbparselex_ver(CharPtr linein, ValNodePtr PNTR lexed, Boolean accver)
 				    current_col[3] == '\0'))
 				{
 				    current_token->choice = GBPARSE_INT_GAP;
-				    current_col += 2;
 				    current_token->data.ptrvalue = MemNew(4);
 				    StringCpy(current_token->data.ptrvalue, "gap");
+				    if(StringNICmp(current_col + 3, "(unk", 4) == 0)
+				    {
+					current_token->choice = GBPARSE_INT_UNK_GAP;
+					last_token = current_token;
+					current_token = ValNodeNew(current_token);
+					current_token->choice = GBPARSE_INT_LEFT;
+					current_col += 4;
+				    }
+				    current_col += 2;
 				    break;
 				}
 				if(StringNCmp(current_col, "gi|", 3) == 0) {
@@ -2559,19 +2621,14 @@ Nlm_gbparselex_ver(CharPtr linein, ValNodePtr PNTR lexed, Boolean accver)
 /* new accessions have .version !!  2/15/1999 */
 				skip_new_token = FALSE;
 				current_token -> choice = GBPARSE_INT_ACCESION;
-				if (IS_ALPHA(*(current_col + 1))) {
-					spare = current_col + 2;
-					dex = 2;
-				} else {
-					spare = current_col + 1;
-					dex = 1;
-				}
-				for (; isdigit(*spare); spare ++){
+				dex = Nlm_gbparse_accprefix(current_col);
+				spare = current_col + dex;
+				for (; isdigit((int) *spare); spare ++){
 					dex ++ ;
 				}
 				if (accver != FALSE && *spare == '.') {
 					dex ++ ;
-					for (spare++; isdigit(*spare); spare ++){
+					for (spare++; isdigit((int) *spare); spare ++){
 						dex ++ ;
 					}
 				}
@@ -2616,7 +2673,7 @@ Nlm_gbparselex_ver(CharPtr linein, ValNodePtr PNTR lexed, Boolean accver)
 NLM_EXTERN CharPtr
 Nlm_non_white(CharPtr ch)
 {
-   while (isspace(*++ch))if (! *ch) break;
+   while (isspace((int) *++ch))if (! *ch) break;
       ;
    return ch;
 }
