@@ -9,8 +9,8 @@
 #include <asn.h>
 #endif
 
-static char * asnfilename = "asnfeat.h15";
-static AsnValxNode avnx[203] = {
+static char * asnfilename = "asnfeat.h18";
+static AsnValxNode avnx[207] = {
     {2,NULL,0,0.0,NULL } ,
     {20,"other" ,0,0.0,&avnx[2] } ,
     {20,"family" ,1,0.0,&avnx[3] } ,
@@ -49,170 +49,174 @@ static AsnValxNode avnx[203] = {
     {20,"gb-acronym" ,32,0.0,&avnx[36] } ,
     {20,"gb-anamorph" ,33,0.0,&avnx[37] } ,
     {20,"gb-synonym" ,34,0.0,&avnx[38] } ,
-    {20,"old-lineage" ,253,0.0,&avnx[39] } ,
-    {20,"old-name" ,254,0.0,&avnx[40] } ,
+    {20,"culture-collection" ,35,0.0,&avnx[39] } ,
+    {20,"bio-material" ,36,0.0,&avnx[40] } ,
+    {20,"metagenome-source" ,37,0.0,&avnx[41] } ,
+    {20,"old-lineage" ,253,0.0,&avnx[42] } ,
+    {20,"old-name" ,254,0.0,&avnx[43] } ,
     {20,"other" ,255,0.0,NULL } ,
-    {20,"not-set" ,0,0.0,&avnx[42] } ,
-    {20,"one" ,1,0.0,&avnx[43] } ,
-    {20,"two" ,2,0.0,&avnx[44] } ,
+    {20,"not-set" ,0,0.0,&avnx[45] } ,
+    {20,"one" ,1,0.0,&avnx[46] } ,
+    {20,"two" ,2,0.0,&avnx[47] } ,
     {20,"three" ,3,0.0,NULL } ,
     {3,NULL,0,0.0,NULL } ,
-    {20,"not-set" ,0,0.0,&avnx[47] } ,
-    {20,"preprotein" ,1,0.0,&avnx[48] } ,
-    {20,"mature" ,2,0.0,&avnx[49] } ,
-    {20,"signal-peptide" ,3,0.0,&avnx[50] } ,
+    {20,"not-set" ,0,0.0,&avnx[50] } ,
+    {20,"preprotein" ,1,0.0,&avnx[51] } ,
+    {20,"mature" ,2,0.0,&avnx[52] } ,
+    {20,"signal-peptide" ,3,0.0,&avnx[53] } ,
     {20,"transit-peptide" ,4,0.0,NULL } ,
     {3,NULL,0,0.0,NULL } ,
-    {20,"unknown" ,0,0.0,&avnx[53] } ,
-    {20,"premsg" ,1,0.0,&avnx[54] } ,
-    {20,"mRNA" ,2,0.0,&avnx[55] } ,
-    {20,"tRNA" ,3,0.0,&avnx[56] } ,
-    {20,"rRNA" ,4,0.0,&avnx[57] } ,
-    {20,"snRNA" ,5,0.0,&avnx[58] } ,
-    {20,"scRNA" ,6,0.0,&avnx[59] } ,
-    {20,"snoRNA" ,7,0.0,&avnx[60] } ,
+    {20,"unknown" ,0,0.0,&avnx[56] } ,
+    {20,"premsg" ,1,0.0,&avnx[57] } ,
+    {20,"mRNA" ,2,0.0,&avnx[58] } ,
+    {20,"tRNA" ,3,0.0,&avnx[59] } ,
+    {20,"rRNA" ,4,0.0,&avnx[60] } ,
+    {20,"snRNA" ,5,0.0,&avnx[61] } ,
+    {20,"scRNA" ,6,0.0,&avnx[62] } ,
+    {20,"snoRNA" ,7,0.0,&avnx[63] } ,
     {20,"other" ,255,0.0,NULL } ,
-    {20,"disulfide" ,1,0.0,&avnx[62] } ,
-    {20,"thiolester" ,2,0.0,&avnx[63] } ,
-    {20,"xlink" ,3,0.0,&avnx[64] } ,
-    {20,"thioether" ,4,0.0,&avnx[65] } ,
+    {20,"disulfide" ,1,0.0,&avnx[65] } ,
+    {20,"thiolester" ,2,0.0,&avnx[66] } ,
+    {20,"xlink" ,3,0.0,&avnx[67] } ,
+    {20,"thioether" ,4,0.0,&avnx[68] } ,
     {20,"other" ,255,0.0,NULL } ,
-    {20,"active" ,1,0.0,&avnx[67] } ,
-    {20,"binding" ,2,0.0,&avnx[68] } ,
-    {20,"cleavage" ,3,0.0,&avnx[69] } ,
-    {20,"inhibit" ,4,0.0,&avnx[70] } ,
-    {20,"modified" ,5,0.0,&avnx[71] } ,
-    {20,"glycosylation" ,6,0.0,&avnx[72] } ,
-    {20,"myristoylation" ,7,0.0,&avnx[73] } ,
-    {20,"mutagenized" ,8,0.0,&avnx[74] } ,
-    {20,"metal-binding" ,9,0.0,&avnx[75] } ,
-    {20,"phosphorylation" ,10,0.0,&avnx[76] } ,
-    {20,"acetylation" ,11,0.0,&avnx[77] } ,
-    {20,"amidation" ,12,0.0,&avnx[78] } ,
-    {20,"methylation" ,13,0.0,&avnx[79] } ,
-    {20,"hydroxylation" ,14,0.0,&avnx[80] } ,
-    {20,"sulfatation" ,15,0.0,&avnx[81] } ,
-    {20,"oxidative-deamination" ,16,0.0,&avnx[82] } ,
-    {20,"pyrrolidone-carboxylic-acid" ,17,0.0,&avnx[83] } ,
-    {20,"gamma-carboxyglutamic-acid" ,18,0.0,&avnx[84] } ,
-    {20,"blocked" ,19,0.0,&avnx[85] } ,
-    {20,"lipid-binding" ,20,0.0,&avnx[86] } ,
-    {20,"np-binding" ,21,0.0,&avnx[87] } ,
-    {20,"dna-binding" ,22,0.0,&avnx[88] } ,
-    {20,"signal-peptide" ,23,0.0,&avnx[89] } ,
-    {20,"transit-peptide" ,24,0.0,&avnx[90] } ,
-    {20,"transmembrane-region" ,25,0.0,&avnx[91] } ,
-    {20,"nitrosylation" ,26,0.0,&avnx[92] } ,
+    {20,"active" ,1,0.0,&avnx[70] } ,
+    {20,"binding" ,2,0.0,&avnx[71] } ,
+    {20,"cleavage" ,3,0.0,&avnx[72] } ,
+    {20,"inhibit" ,4,0.0,&avnx[73] } ,
+    {20,"modified" ,5,0.0,&avnx[74] } ,
+    {20,"glycosylation" ,6,0.0,&avnx[75] } ,
+    {20,"myristoylation" ,7,0.0,&avnx[76] } ,
+    {20,"mutagenized" ,8,0.0,&avnx[77] } ,
+    {20,"metal-binding" ,9,0.0,&avnx[78] } ,
+    {20,"phosphorylation" ,10,0.0,&avnx[79] } ,
+    {20,"acetylation" ,11,0.0,&avnx[80] } ,
+    {20,"amidation" ,12,0.0,&avnx[81] } ,
+    {20,"methylation" ,13,0.0,&avnx[82] } ,
+    {20,"hydroxylation" ,14,0.0,&avnx[83] } ,
+    {20,"sulfatation" ,15,0.0,&avnx[84] } ,
+    {20,"oxidative-deamination" ,16,0.0,&avnx[85] } ,
+    {20,"pyrrolidone-carboxylic-acid" ,17,0.0,&avnx[86] } ,
+    {20,"gamma-carboxyglutamic-acid" ,18,0.0,&avnx[87] } ,
+    {20,"blocked" ,19,0.0,&avnx[88] } ,
+    {20,"lipid-binding" ,20,0.0,&avnx[89] } ,
+    {20,"np-binding" ,21,0.0,&avnx[90] } ,
+    {20,"dna-binding" ,22,0.0,&avnx[91] } ,
+    {20,"signal-peptide" ,23,0.0,&avnx[92] } ,
+    {20,"transit-peptide" ,24,0.0,&avnx[93] } ,
+    {20,"transmembrane-region" ,25,0.0,&avnx[94] } ,
+    {20,"nitrosylation" ,26,0.0,&avnx[95] } ,
     {20,"other" ,255,0.0,NULL } ,
-    {20,"unknown" ,0,0.0,&avnx[94] } ,
-    {20,"pol1" ,1,0.0,&avnx[95] } ,
-    {20,"pol2" ,2,0.0,&avnx[96] } ,
-    {20,"pol3" ,3,0.0,&avnx[97] } ,
-    {20,"bacterial" ,4,0.0,&avnx[98] } ,
-    {20,"viral" ,5,0.0,&avnx[99] } ,
-    {20,"rna" ,6,0.0,&avnx[100] } ,
-    {20,"organelle" ,7,0.0,&avnx[101] } ,
+    {20,"unknown" ,0,0.0,&avnx[97] } ,
+    {20,"pol1" ,1,0.0,&avnx[98] } ,
+    {20,"pol2" ,2,0.0,&avnx[99] } ,
+    {20,"pol3" ,3,0.0,&avnx[100] } ,
+    {20,"bacterial" ,4,0.0,&avnx[101] } ,
+    {20,"viral" ,5,0.0,&avnx[102] } ,
+    {20,"rna" ,6,0.0,&avnx[103] } ,
+    {20,"organelle" ,7,0.0,&avnx[104] } ,
     {20,"other" ,255,0.0,NULL } ,
     {2,NULL,0,0.0,NULL } ,
     {2,NULL,0,0.0,NULL } ,
-    {20,"unknown" ,0,0.0,&avnx[105] } ,
-    {20,"single" ,1,0.0,&avnx[106] } ,
-    {20,"multiple" ,2,0.0,&avnx[107] } ,
+    {20,"unknown" ,0,0.0,&avnx[108] } ,
+    {20,"single" ,1,0.0,&avnx[109] } ,
+    {20,"multiple" ,2,0.0,&avnx[110] } ,
     {20,"region" ,3,0.0,NULL } ,
-    {20,"unknown" ,0,0.0,&avnx[109] } ,
-    {20,"rna-seq" ,1,0.0,&avnx[110] } ,
-    {20,"rna-size" ,2,0.0,&avnx[111] } ,
-    {20,"np-map" ,3,0.0,&avnx[112] } ,
-    {20,"np-size" ,4,0.0,&avnx[113] } ,
-    {20,"pe-seq" ,5,0.0,&avnx[114] } ,
-    {20,"cDNA-seq" ,6,0.0,&avnx[115] } ,
-    {20,"pe-map" ,7,0.0,&avnx[116] } ,
-    {20,"pe-size" ,8,0.0,&avnx[117] } ,
-    {20,"pseudo-seq" ,9,0.0,&avnx[118] } ,
-    {20,"rev-pe-map" ,10,0.0,&avnx[119] } ,
+    {20,"unknown" ,0,0.0,&avnx[112] } ,
+    {20,"rna-seq" ,1,0.0,&avnx[113] } ,
+    {20,"rna-size" ,2,0.0,&avnx[114] } ,
+    {20,"np-map" ,3,0.0,&avnx[115] } ,
+    {20,"np-size" ,4,0.0,&avnx[116] } ,
+    {20,"pe-seq" ,5,0.0,&avnx[117] } ,
+    {20,"cDNA-seq" ,6,0.0,&avnx[118] } ,
+    {20,"pe-map" ,7,0.0,&avnx[119] } ,
+    {20,"pe-size" ,8,0.0,&avnx[120] } ,
+    {20,"pseudo-seq" ,9,0.0,&avnx[121] } ,
+    {20,"rev-pe-map" ,10,0.0,&avnx[122] } ,
     {20,"other" ,255,0.0,NULL } ,
-    {20,"unknown" ,0,0.0,&avnx[121] } ,
-    {20,"physiological" ,1,0.0,&avnx[122] } ,
-    {20,"in-vitro" ,2,0.0,&avnx[123] } ,
-    {20,"oocyte" ,3,0.0,&avnx[124] } ,
-    {20,"transfection" ,4,0.0,&avnx[125] } ,
-    {20,"transgenic" ,5,0.0,&avnx[126] } ,
+    {20,"unknown" ,0,0.0,&avnx[124] } ,
+    {20,"physiological" ,1,0.0,&avnx[125] } ,
+    {20,"in-vitro" ,2,0.0,&avnx[126] } ,
+    {20,"oocyte" ,3,0.0,&avnx[127] } ,
+    {20,"transfection" ,4,0.0,&avnx[128] } ,
+    {20,"transgenic" ,5,0.0,&avnx[129] } ,
     {20,"other" ,255,0.0,NULL } ,
     {3,NULL,1,0.0,NULL } ,
     {2,NULL,0,0.0,NULL } ,
     {2,NULL,0,0.0,NULL } ,
-    {20,"helix" ,1,0.0,&avnx[131] } ,
-    {20,"sheet" ,2,0.0,&avnx[132] } ,
+    {20,"helix" ,1,0.0,&avnx[134] } ,
+    {20,"sheet" ,2,0.0,&avnx[135] } ,
     {20,"turn" ,3,0.0,NULL } ,
-    {20,"unknown" ,0,0.0,&avnx[134] } ,
-    {20,"genomic" ,1,0.0,&avnx[135] } ,
-    {20,"chloroplast" ,2,0.0,&avnx[136] } ,
-    {20,"chromoplast" ,3,0.0,&avnx[137] } ,
-    {20,"kinetoplast" ,4,0.0,&avnx[138] } ,
-    {20,"mitochondrion" ,5,0.0,&avnx[139] } ,
-    {20,"plastid" ,6,0.0,&avnx[140] } ,
-    {20,"macronuclear" ,7,0.0,&avnx[141] } ,
-    {20,"extrachrom" ,8,0.0,&avnx[142] } ,
-    {20,"plasmid" ,9,0.0,&avnx[143] } ,
-    {20,"transposon" ,10,0.0,&avnx[144] } ,
-    {20,"insertion-seq" ,11,0.0,&avnx[145] } ,
-    {20,"cyanelle" ,12,0.0,&avnx[146] } ,
-    {20,"proviral" ,13,0.0,&avnx[147] } ,
-    {20,"virion" ,14,0.0,&avnx[148] } ,
-    {20,"nucleomorph" ,15,0.0,&avnx[149] } ,
-    {20,"apicoplast" ,16,0.0,&avnx[150] } ,
-    {20,"leucoplast" ,17,0.0,&avnx[151] } ,
-    {20,"proplastid" ,18,0.0,&avnx[152] } ,
-    {20,"endogenous-virus" ,19,0.0,&avnx[153] } ,
-    {20,"hydrogenosome" ,20,0.0,&avnx[154] } ,
+    {20,"unknown" ,0,0.0,&avnx[137] } ,
+    {20,"genomic" ,1,0.0,&avnx[138] } ,
+    {20,"chloroplast" ,2,0.0,&avnx[139] } ,
+    {20,"chromoplast" ,3,0.0,&avnx[140] } ,
+    {20,"kinetoplast" ,4,0.0,&avnx[141] } ,
+    {20,"mitochondrion" ,5,0.0,&avnx[142] } ,
+    {20,"plastid" ,6,0.0,&avnx[143] } ,
+    {20,"macronuclear" ,7,0.0,&avnx[144] } ,
+    {20,"extrachrom" ,8,0.0,&avnx[145] } ,
+    {20,"plasmid" ,9,0.0,&avnx[146] } ,
+    {20,"transposon" ,10,0.0,&avnx[147] } ,
+    {20,"insertion-seq" ,11,0.0,&avnx[148] } ,
+    {20,"cyanelle" ,12,0.0,&avnx[149] } ,
+    {20,"proviral" ,13,0.0,&avnx[150] } ,
+    {20,"virion" ,14,0.0,&avnx[151] } ,
+    {20,"nucleomorph" ,15,0.0,&avnx[152] } ,
+    {20,"apicoplast" ,16,0.0,&avnx[153] } ,
+    {20,"leucoplast" ,17,0.0,&avnx[154] } ,
+    {20,"proplastid" ,18,0.0,&avnx[155] } ,
+    {20,"endogenous-virus" ,19,0.0,&avnx[156] } ,
+    {20,"hydrogenosome" ,20,0.0,&avnx[157] } ,
     {20,"chromosome" ,21,0.0,NULL } ,
     {3,NULL,0,0.0,NULL } ,
-    {20,"unknown" ,0,0.0,&avnx[157] } ,
-    {20,"natural" ,1,0.0,&avnx[158] } ,
-    {20,"natmut" ,2,0.0,&avnx[159] } ,
-    {20,"mut" ,3,0.0,&avnx[160] } ,
-    {20,"artificial" ,4,0.0,&avnx[161] } ,
-    {20,"synthetic" ,5,0.0,&avnx[162] } ,
+    {20,"unknown" ,0,0.0,&avnx[160] } ,
+    {20,"natural" ,1,0.0,&avnx[161] } ,
+    {20,"natmut" ,2,0.0,&avnx[162] } ,
+    {20,"mut" ,3,0.0,&avnx[163] } ,
+    {20,"artificial" ,4,0.0,&avnx[164] } ,
+    {20,"synthetic" ,5,0.0,&avnx[165] } ,
     {20,"other" ,255,0.0,NULL } ,
     {3,NULL,0,0.0,NULL } ,
-    {20,"chromosome" ,1,0.0,&avnx[165] } ,
-    {20,"map" ,2,0.0,&avnx[166] } ,
-    {20,"clone" ,3,0.0,&avnx[167] } ,
-    {20,"subclone" ,4,0.0,&avnx[168] } ,
-    {20,"haplotype" ,5,0.0,&avnx[169] } ,
-    {20,"genotype" ,6,0.0,&avnx[170] } ,
-    {20,"sex" ,7,0.0,&avnx[171] } ,
-    {20,"cell-line" ,8,0.0,&avnx[172] } ,
-    {20,"cell-type" ,9,0.0,&avnx[173] } ,
-    {20,"tissue-type" ,10,0.0,&avnx[174] } ,
-    {20,"clone-lib" ,11,0.0,&avnx[175] } ,
-    {20,"dev-stage" ,12,0.0,&avnx[176] } ,
-    {20,"frequency" ,13,0.0,&avnx[177] } ,
-    {20,"germline" ,14,0.0,&avnx[178] } ,
-    {20,"rearranged" ,15,0.0,&avnx[179] } ,
-    {20,"lab-host" ,16,0.0,&avnx[180] } ,
-    {20,"pop-variant" ,17,0.0,&avnx[181] } ,
-    {20,"tissue-lib" ,18,0.0,&avnx[182] } ,
-    {20,"plasmid-name" ,19,0.0,&avnx[183] } ,
-    {20,"transposon-name" ,20,0.0,&avnx[184] } ,
-    {20,"insertion-seq-name" ,21,0.0,&avnx[185] } ,
-    {20,"plastid-name" ,22,0.0,&avnx[186] } ,
-    {20,"country" ,23,0.0,&avnx[187] } ,
-    {20,"segment" ,24,0.0,&avnx[188] } ,
-    {20,"endogenous-virus-name" ,25,0.0,&avnx[189] } ,
-    {20,"transgenic" ,26,0.0,&avnx[190] } ,
-    {20,"environmental-sample" ,27,0.0,&avnx[191] } ,
-    {20,"isolation-source" ,28,0.0,&avnx[192] } ,
-    {20,"lat-lon" ,29,0.0,&avnx[193] } ,
-    {20,"collection-date" ,30,0.0,&avnx[194] } ,
-    {20,"collected-by" ,31,0.0,&avnx[195] } ,
-    {20,"identified-by" ,32,0.0,&avnx[196] } ,
-    {20,"fwd-primer-seq" ,33,0.0,&avnx[197] } ,
-    {20,"rev-primer-seq" ,34,0.0,&avnx[198] } ,
-    {20,"fwd-primer-name" ,35,0.0,&avnx[199] } ,
-    {20,"rev-primer-name" ,36,0.0,&avnx[200] } ,
+    {20,"chromosome" ,1,0.0,&avnx[168] } ,
+    {20,"map" ,2,0.0,&avnx[169] } ,
+    {20,"clone" ,3,0.0,&avnx[170] } ,
+    {20,"subclone" ,4,0.0,&avnx[171] } ,
+    {20,"haplotype" ,5,0.0,&avnx[172] } ,
+    {20,"genotype" ,6,0.0,&avnx[173] } ,
+    {20,"sex" ,7,0.0,&avnx[174] } ,
+    {20,"cell-line" ,8,0.0,&avnx[175] } ,
+    {20,"cell-type" ,9,0.0,&avnx[176] } ,
+    {20,"tissue-type" ,10,0.0,&avnx[177] } ,
+    {20,"clone-lib" ,11,0.0,&avnx[178] } ,
+    {20,"dev-stage" ,12,0.0,&avnx[179] } ,
+    {20,"frequency" ,13,0.0,&avnx[180] } ,
+    {20,"germline" ,14,0.0,&avnx[181] } ,
+    {20,"rearranged" ,15,0.0,&avnx[182] } ,
+    {20,"lab-host" ,16,0.0,&avnx[183] } ,
+    {20,"pop-variant" ,17,0.0,&avnx[184] } ,
+    {20,"tissue-lib" ,18,0.0,&avnx[185] } ,
+    {20,"plasmid-name" ,19,0.0,&avnx[186] } ,
+    {20,"transposon-name" ,20,0.0,&avnx[187] } ,
+    {20,"insertion-seq-name" ,21,0.0,&avnx[188] } ,
+    {20,"plastid-name" ,22,0.0,&avnx[189] } ,
+    {20,"country" ,23,0.0,&avnx[190] } ,
+    {20,"segment" ,24,0.0,&avnx[191] } ,
+    {20,"endogenous-virus-name" ,25,0.0,&avnx[192] } ,
+    {20,"transgenic" ,26,0.0,&avnx[193] } ,
+    {20,"environmental-sample" ,27,0.0,&avnx[194] } ,
+    {20,"isolation-source" ,28,0.0,&avnx[195] } ,
+    {20,"lat-lon" ,29,0.0,&avnx[196] } ,
+    {20,"collection-date" ,30,0.0,&avnx[197] } ,
+    {20,"collected-by" ,31,0.0,&avnx[198] } ,
+    {20,"identified-by" ,32,0.0,&avnx[199] } ,
+    {20,"fwd-primer-seq" ,33,0.0,&avnx[200] } ,
+    {20,"rev-primer-seq" ,34,0.0,&avnx[201] } ,
+    {20,"fwd-primer-name" ,35,0.0,&avnx[202] } ,
+    {20,"rev-primer-name" ,36,0.0,&avnx[203] } ,
+    {20,"metagenomic" ,37,0.0,&avnx[204] } ,
     {20,"other" ,255,0.0,NULL } ,
-    {20,"experimental" ,1,0.0,&avnx[202] } ,
+    {20,"experimental" ,1,0.0,&avnx[206] } ,
     {20,"not-experimental" ,2,0.0,NULL } };
 
 static AsnType atx[238] = {
@@ -295,7 +299,7 @@ static AsnType atx[238] = {
   {0, "cdregion" ,128,2,0,0,0,0,0,0,NULL,&atx[77],NULL,0,&atx[106]} ,
   {423, "Cdregion" ,1,0,0,0,0,0,0,0,NULL,&atx[31],&atx[78],0,&atx[142]} ,
   {0, "orf" ,128,0,0,1,0,0,0,0,NULL,&atx[23],NULL,0,&atx[79]} ,
-  {0, "frame" ,128,1,0,0,1,0,0,0,&avnx[45],&atx[80],&avnx[41],0,&atx[81]} ,
+  {0, "frame" ,128,1,0,0,1,0,0,0,&avnx[48],&atx[80],&avnx[44],0,&atx[81]} ,
   {310, "ENUMERATED" ,0,10,0,0,0,0,0,0,NULL,NULL,NULL,0,NULL} ,
   {0, "conflict" ,128,2,0,1,0,0,0,0,NULL,&atx[23],NULL,0,&atx[82]} ,
   {0, "gaps" ,128,3,0,1,0,0,0,0,NULL,&atx[4],NULL,0,&atx[83]} ,
@@ -335,11 +339,11 @@ static AsnType atx[238] = {
   {0, "db" ,128,4,0,1,0,0,0,0,NULL,&atx[27],&atx[117],0,&atx[119]} ,
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[118],NULL,0,NULL} ,
   {402, "Dbtag" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,NULL} ,
-  {0, "processed" ,128,5,0,0,1,0,0,0,&avnx[51],&atx[80],&avnx[46],0,NULL} ,
+  {0, "processed" ,128,5,0,0,1,0,0,0,&avnx[54],&atx[80],&avnx[49],0,NULL} ,
   {0, "rna" ,128,4,0,0,0,0,0,0,NULL,&atx[121],NULL,0,&atx[138]} ,
   {408, "RNA-ref" ,1,0,0,0,0,0,1,0,NULL,&atx[122],NULL,0,&atx[100]} ,
   {401, "RNA-ref" ,1,0,0,0,0,1,0,0,NULL,&atx[31],&atx[123],0,&atx[128]} ,
-  {0, "type" ,128,0,0,0,0,0,0,0,NULL,&atx[80],&avnx[52],0,&atx[124]} ,
+  {0, "type" ,128,0,0,0,0,0,0,0,NULL,&atx[80],&avnx[55],0,&atx[124]} ,
   {0, "pseudo" ,128,1,0,1,0,0,0,0,NULL,&atx[23],NULL,0,&atx[125]} ,
   {0, "ext" ,128,2,0,1,0,0,0,0,NULL,&atx[11],&atx[126],0,NULL} ,
   {0, "name" ,128,0,0,0,0,0,0,0,NULL,&atx[18],NULL,0,&atx[127]} ,
@@ -365,8 +369,8 @@ static AsnType atx[238] = {
   {0, "region" ,128,8,0,0,0,0,0,0,NULL,&atx[18],NULL,0,&atx[147]} ,
   {0, "comment" ,128,9,0,0,0,0,0,0,NULL,&atx[148],NULL,0,&atx[149]} ,
   {305, "NULL" ,0,5,0,0,0,0,0,0,NULL,NULL,NULL,0,NULL} ,
-  {0, "bond" ,128,10,0,0,0,0,0,0,NULL,&atx[80],&avnx[61],0,&atx[150]} ,
-  {0, "site" ,128,11,0,0,0,0,0,0,NULL,&atx[80],&avnx[66],0,&atx[151]} ,
+  {0, "bond" ,128,10,0,0,0,0,0,0,NULL,&atx[80],&avnx[64],0,&atx[150]} ,
+  {0, "site" ,128,11,0,0,0,0,0,0,NULL,&atx[80],&avnx[69],0,&atx[151]} ,
   {0, "rsite" ,128,12,0,0,0,0,0,0,NULL,&atx[152],NULL,0,&atx[157]} ,
   {414, "Rsite-ref" ,1,0,0,0,0,0,1,0,NULL,&atx[153],NULL,0,&atx[160]} ,
   {401, "Rsite-ref" ,1,0,0,0,0,1,0,0,NULL,&atx[11],&atx[154],0,&atx[156]} ,
@@ -390,37 +394,37 @@ static AsnType atx[238] = {
   {0, "rna" ,128,4,0,1,0,0,0,0,NULL,&atx[56],&atx[172],0,&atx[173]} ,
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[18],NULL,0,NULL} ,
   {0, "expression" ,128,5,0,1,0,0,0,0,NULL,&atx[18],NULL,0,&atx[174]} ,
-  {0, "txsystem" ,128,6,0,0,0,0,0,0,NULL,&atx[80],&avnx[93],0,&atx[175]} ,
+  {0, "txsystem" ,128,6,0,0,0,0,0,0,NULL,&atx[80],&avnx[96],0,&atx[175]} ,
   {0, "txdescr" ,128,7,0,1,0,0,0,0,NULL,&atx[18],NULL,0,&atx[176]} ,
   {0, "txorg" ,128,8,0,1,0,0,0,0,NULL,&atx[177],NULL,0,&atx[178]} ,
   {404, "Org-ref" ,1,0,0,0,0,0,1,0,NULL,&atx[34],NULL,0,&atx[183]} ,
-  {0, "mapping-precise" ,128,9,0,0,1,0,0,0,&avnx[102],&atx[23],NULL,0,&atx[179]} ,
-  {0, "location-accurate" ,128,10,0,0,1,0,0,0,&avnx[103],&atx[23],NULL,0,&atx[180]} ,
-  {0, "inittype" ,128,11,0,1,0,0,0,0,NULL,&atx[80],&avnx[104],0,&atx[181]} ,
+  {0, "mapping-precise" ,128,9,0,0,1,0,0,0,&avnx[105],&atx[23],NULL,0,&atx[179]} ,
+  {0, "location-accurate" ,128,10,0,0,1,0,0,0,&avnx[106],&atx[23],NULL,0,&atx[180]} ,
+  {0, "inittype" ,128,11,0,1,0,0,0,0,NULL,&atx[80],&avnx[107],0,&atx[181]} ,
   {0, "evidence" ,128,12,0,1,0,0,0,0,NULL,&atx[27],&atx[182],0,NULL} ,
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[183],NULL,0,NULL} ,
   {405, "Tx-evidence" ,1,0,0,0,0,0,0,0,NULL,&atx[31],&atx[184],0,NULL} ,
-  {0, "exp-code" ,128,0,0,0,0,0,0,0,NULL,&atx[80],&avnx[108],0,&atx[185]} ,
-  {0, "expression-system" ,128,1,0,0,1,0,0,0,&avnx[127],&atx[80],&avnx[120],0,&atx[186]} ,
-  {0, "low-prec-data" ,128,2,0,0,1,0,0,0,&avnx[128],&atx[23],NULL,0,&atx[187]} ,
-  {0, "from-homolog" ,128,3,0,0,1,0,0,0,&avnx[129],&atx[23],NULL,0,NULL} ,
+  {0, "exp-code" ,128,0,0,0,0,0,0,0,NULL,&atx[80],&avnx[111],0,&atx[185]} ,
+  {0, "expression-system" ,128,1,0,0,1,0,0,0,&avnx[130],&atx[80],&avnx[123],0,&atx[186]} ,
+  {0, "low-prec-data" ,128,2,0,0,1,0,0,0,&avnx[131],&atx[23],NULL,0,&atx[187]} ,
+  {0, "from-homolog" ,128,3,0,0,1,0,0,0,&avnx[132],&atx[23],NULL,0,NULL} ,
   {0, "num" ,128,15,0,0,0,0,0,0,NULL,&atx[189],NULL,0,&atx[190]} ,
   {412, "Numbering" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[193]} ,
-  {0, "psec-str" ,128,16,0,0,0,0,0,0,NULL,&atx[80],&avnx[130],0,&atx[191]} ,
+  {0, "psec-str" ,128,16,0,0,0,0,0,0,NULL,&atx[80],&avnx[133],0,&atx[191]} ,
   {0, "non-std-residue" ,128,17,0,0,0,0,0,0,NULL,&atx[18],NULL,0,&atx[192]} ,
   {0, "het" ,128,18,0,0,0,0,0,0,NULL,&atx[193],NULL,0,&atx[194]} ,
   {413, "Heterogen" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[152]} ,
   {0, "biosrc" ,128,19,0,0,0,0,0,0,NULL,&atx[195],NULL,0,NULL} ,
   {407, "BioSource" ,1,0,0,0,0,0,1,0,NULL,&atx[196],NULL,0,&atx[121]} ,
   {401, "BioSource" ,1,0,0,0,0,1,0,0,NULL,&atx[31],&atx[197],0,&atx[200]} ,
-  {0, "genome" ,128,0,0,0,1,0,0,0,&avnx[155],&atx[4],&avnx[133],0,&atx[198]} ,
-  {0, "origin" ,128,1,0,0,1,0,0,0,&avnx[163],&atx[4],&avnx[156],0,&atx[199]} ,
+  {0, "genome" ,128,0,0,0,1,0,0,0,&avnx[158],&atx[4],&avnx[136],0,&atx[198]} ,
+  {0, "origin" ,128,1,0,0,1,0,0,0,&avnx[166],&atx[4],&avnx[159],0,&atx[199]} ,
   {0, "org" ,128,2,0,0,0,0,0,0,NULL,&atx[200],NULL,0,&atx[201]} ,
   {402, "Org-ref" ,1,0,0,0,0,0,1,0,NULL,&atx[34],NULL,0,&atx[203]} ,
   {0, "subtype" ,128,3,0,1,0,0,0,0,NULL,&atx[56],&atx[202],0,&atx[207]} ,
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[203],NULL,0,NULL} ,
   {403, "SubSource" ,1,0,0,0,0,0,0,0,NULL,&atx[31],&atx[204],0,NULL} ,
-  {0, "subtype" ,128,0,0,0,0,0,0,0,NULL,&atx[4],&avnx[164],0,&atx[205]} ,
+  {0, "subtype" ,128,0,0,0,0,0,0,0,NULL,&atx[4],&avnx[167],0,&atx[205]} ,
   {0, "name" ,128,1,0,0,0,0,0,0,NULL,&atx[18],NULL,0,&atx[206]} ,
   {0, "attrib" ,128,2,0,1,0,0,0,0,NULL,&atx[18],NULL,0,NULL} ,
   {0, "is-focus" ,128,4,0,1,0,0,0,0,NULL,&atx[148],NULL,0,NULL} ,
@@ -438,7 +442,7 @@ static AsnType atx[238] = {
   {0, "ext" ,128,9,0,1,0,0,0,0,NULL,&atx[158],NULL,0,&atx[220]} ,
   {0, "cit" ,128,10,0,1,0,0,0,0,NULL,&atx[221],NULL,0,&atx[222]} ,
   {416, "Pub-set" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[8]} ,
-  {0, "exp-ev" ,128,11,0,1,0,0,0,0,NULL,&atx[80],&avnx[201],0,&atx[223]} ,
+  {0, "exp-ev" ,128,11,0,1,0,0,0,0,NULL,&atx[80],&avnx[205],0,&atx[223]} ,
   {0, "xref" ,128,12,0,1,0,0,0,0,NULL,&atx[27],&atx[224],0,&atx[228]} ,
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[225],NULL,0,NULL} ,
   {422, "SeqFeatXref" ,1,0,0,0,0,0,0,0,NULL,&atx[31],&atx[226],0,&atx[77]} ,
@@ -456,7 +460,7 @@ static AsnType atx[238] = {
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[85],NULL,0,NULL} };
 
 static AsnModule ampx[8] = {
-  { "NCBI-Seqfeat" , "asnfeat.h15",&atx[0],NULL,&ampx[1],0,0} ,
+  { "NCBI-Seqfeat" , "asnfeat.h18",&atx[0],NULL,&ampx[1],0,0} ,
   { "NCBI-Rsite" , NULL,&atx[153],NULL,&ampx[2],0,0} ,
   { "NCBI-RNA" , NULL,&atx[122],NULL,&ampx[3],0,0} ,
   { "NCBI-Gene" , NULL,&atx[16],NULL,&ampx[4],0,0} ,
