@@ -1,4 +1,4 @@
-/*  $Id: ncbi_connutil.c,v 6.108 2006/04/21 14:38:12 lavr Exp $
+/*  $Id: ncbi_connutil.c,v 6.115 2007/04/05 12:26:02 kazimird Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -37,27 +37,13 @@
 #include <ctype.h>
 #include <errno.h>
 #include <stdlib.h>
-#if defined(NCBI_OS_UNIX)
-#  ifndef NCBI_OS_SOLARIS
-#    include <limits.h>
-#  endif
-#  if defined(HAVE_GETPWUID)  ||  defined(HAVE_GETPWUID_R)
-#    include <pwd.h>
-#  endif
-#  include <unistd.h>
-#elif defined(NCBI_OS_MSWIN)
-#  if defined(_MSC_VER)  &&  (_MSC_VER > 1200)
-#    define WIN32_LEAN_AND_MEAN
-#  endif
-#  include <windows.h>
-#endif
 
 
 extern const char* ConnNetInfo_GetValue(const char* service, const char* param,
                                         char* value, size_t value_size,
                                         const char* def_value)
 {
-    char        buf[256];
+    char        buf[128];
     const char* val;
     char*       s;
 
@@ -130,7 +116,7 @@ extern SConnNetInfo* ConnNetInfo_Create(const char* service)
                                                 (service  &&  *service
                                                  ? strlen(service) + 1 : 0));
     /* aux. storage for the string-to-int conversions, etc. */
-    char   str[256];
+    char   str[1024];
     int    val;
     double dbl;
     char*  s;
@@ -138,9 +124,8 @@ extern SConnNetInfo* ConnNetInfo_Create(const char* service)
     if (!info)
         return 0/*failure*/;
 
-    /* client host */
-    if (!SOCK_gethostbyaddr(0, info->client_host, sizeof(info->client_host)))
-        SOCK_gethostname(info->client_host, sizeof(info->client_host));
+    /* client host: default */
+    info->client_host[0] = '\0';
 
     /* Future extensions, clear up for now */
     info->scheme  = eURL_Unspec;
@@ -256,6 +241,11 @@ extern SConnNetInfo* ConnNetInfo_Create(const char* service)
     } else
         info->http_user_header = 0;
 
+    /* default referer */
+    ConnNetInfo_GetValue(0, REG_CONN_HTTP_REFERER, str, sizeof(str),
+                         DEF_CONN_HTTP_REFERER);
+    info->http_referer = *str ? strdup(str) : 0;
+
     /* not adjusted yet... */
     info->http_proxy_adjusted = 0/*false*/;
     /* store service name for which this structure has been created */
@@ -514,7 +504,7 @@ static int/*bool*/ s_ModifyUserHeader(SConnNetInfo* info,
             } else
                 l = len;
             if (l != linelen) {
-                size_t off  = (size_t)(line - hdr);
+                size_t off = (size_t)(line - hdr);
                 if (l > linelen) {
                     char* temp = (char*)realloc(hdr, hdrlen + l - linelen + 1);
                     if (!temp) {
@@ -718,41 +708,24 @@ extern int/*bool*/ ConnNetInfo_PostOverrideArg(SConnNetInfo* info,
 
 static int/*bool*/ s_IsSufficientAddress(const char* addr)
 {
-    size_t i, len = strlen(addr);
-    int dots = 0, isip = 1;
-    const char* dot = 0;
-
-    for (i = 0; i < len; i++) {
-        if (addr[i] == '.') {
-            ++dots;
-            if (i  &&  dots <= 3) {
-                if (isip) {
-                    size_t n = (size_t)(&addr[i] - (dot ? dot : addr - 1));
-                    if (n <= 1  ||  n > 4)
-                        isip = 0;
-                }
-            } else
-                isip = 0;
-            dot = &addr[i];
-        } else if (!isdigit((unsigned char) addr[i]))
-            isip = 0;
-    }
-    if (dots < 3)
-        isip = 0;
-    if (dot == &addr[len - 1])
-        --dots;
-    return isip ? 1 : dots < 2 ? 0 : 1;
+    const char*c;
+    return (SOCK_isip(addr)  ||
+            ((c = strchr(addr, '.'))  != 0  &&  c[1]  &&
+             (c = strchr(c + 2, '.')) != 0  &&  c[1]));
 }
 
 
-static const char* s_ClientAddress(const char* client_host)
+static const char* s_ClientAddress(const char* client_host, int/*bool*/ local)
 {
     unsigned int ip;
     char addr[64];
     char* s;
 
-    if (!client_host  ||  s_IsSufficientAddress(client_host)        ||
-        !(ip = SOCK_gethostbyname(*client_host ? client_host : 0))  ||
+    assert(client_host);
+    if (s_IsSufficientAddress(client_host)                          ||
+        !(ip = *client_host  &&  !local
+          ? SOCK_gethostbyname(client_host)
+          : SOCK_GetLocalHostAddress(eDefault))                     ||
         SOCK_ntoa(ip, addr, sizeof(addr)) != 0                      ||
         !(s = (char*) malloc(strlen(client_host) + strlen(addr) + 3))) {
         return client_host;
@@ -767,6 +740,7 @@ extern int/*bool*/ ConnNetInfo_SetupStandardArgs(SConnNetInfo* info)
     static const char service[]  = "service";
     static const char address[]  = "address";
     static const char platform[] = "platform";
+    int/*bool*/ local;
     const char* arch;
     const char* addr;
 
@@ -776,12 +750,17 @@ extern int/*bool*/ ConnNetInfo_SetupStandardArgs(SConnNetInfo* info)
         assert(0);
         return 0/*failed*/;
     }
-    /* Dispatcher CGI arguments (sacrifice some if they all do not fit) */
+    /* Dispatcher CGI args (may sacrifice some if they don't fit altogether) */
     if (!(arch = CORE_GetPlatform())  ||  !*arch)
         ConnNetInfo_DeleteArg(info, platform);
     else
         ConnNetInfo_PreOverrideArg(info, platform, arch);
-    if (!(addr = s_ClientAddress(info->client_host))  ||  !*addr)
+    local = !info->client_host[0];
+    if (local  &&
+        !SOCK_gethostbyaddr(0, info->client_host, sizeof(info->client_host))) {
+        SOCK_gethostname(info->client_host, sizeof(info->client_host));
+    }
+    if (!(addr = s_ClientAddress(info->client_host, local))  ||  !*addr)
         ConnNetInfo_DeleteArg(info, address);
     else
         ConnNetInfo_PreOverrideArg(info, address, addr);
@@ -820,6 +799,7 @@ extern SConnNetInfo* ConnNetInfo_Clone(const SConnNetInfo* info)
     }
     x_info->http_user_header = 0;
     ConnNetInfo_SetUserHeader(x_info, info->http_user_header);
+    x_info->http_referer = info->http_referer ? strdup(info->http_referer) : 0;
     return x_info;
 }
 
@@ -850,7 +830,9 @@ extern void ConnNetInfo_Log(const SConnNetInfo* info, LOG lg)
     if (!(s = (char*) malloc(sizeof(*info) + 4096 +
                              (info->service ? strlen(info->service) : 0) +
                              (info->http_user_header
-                              ? strlen(info->http_user_header) : 0)))) {
+                              ? strlen(info->http_user_header) : 0) +
+                             (info->http_referer
+                              ? strlen(info->http_referer) : 0)))) {
         LOG_WRITE(lg, eLOG_Error, "ConnNetInfo_Log: Cannot alloc temp buffer");
         return;
     }
@@ -858,7 +840,9 @@ extern void ConnNetInfo_Log(const SConnNetInfo* info, LOG lg)
     strcpy(s, "ConnNetInfo_Log\n"
            "#################### [BEGIN] SConnNetInfo:\n");
     s_SaveString    (s, "service",         info->service);
-    s_SaveString    (s, "client_host",     info->client_host);
+    s_SaveString    (s, "client_host",     (*info->client_host
+                                            ? info->client_host
+                                            : "<default>"));
     s_SaveString    (s, "host",            info->host);
     s_SaveULong     (s, "port",            info->port);
     s_SaveString    (s, "path",            info->path);
@@ -892,7 +876,8 @@ extern void ConnNetInfo_Log(const SConnNetInfo* info, LOG lg)
     s_SaveBool      (s, "stateless",       info->stateless);
     s_SaveBool      (s, "firewall",        info->firewall);
     s_SaveBool      (s, "lb_disable",      info->lb_disable);
-    s_SaveString    (s, "user_header",     info->http_user_header);
+    s_SaveString    (s, "http_user_header",info->http_user_header);
+    s_SaveString    (s, "http_referer",    info->http_referer);
     s_SaveBool      (s, "proxy_adjusted",  info->http_proxy_adjusted);
     strcat(s, "#################### [END] SConnNetInfo\n");
 
@@ -906,6 +891,10 @@ extern void ConnNetInfo_Destroy(SConnNetInfo* info)
     if (!info)
         return;
     ConnNetInfo_SetUserHeader(info, 0);
+    if (info->http_referer) {
+        free((void*) info->http_referer);
+        info->http_referer = 0;
+    }
     free(info);
 }
 
@@ -1278,13 +1267,11 @@ extern EIO_Status BUF_StripToPattern
  */
 static int s_HexChar(char ch)
 {
-    if ('0' <= ch  &&  ch <= '9')
-        return ch - '0';
-    if ('a' <= ch  &&  ch <= 'f')
-        return 10 + (ch - 'a');
-    if ('A' <= ch  &&  ch <= 'F')
-        return 10 + (ch - 'A');
-    return -1;
+    unsigned int rc = ch - '0';
+    if (rc <= 9)
+        return rc;
+    rc = (ch | ' ') - 'a';
+    return rc <= 5 ? (int) rc + 10 : -1;
 }
 
 
@@ -1793,563 +1780,3 @@ extern size_t HostPortToString(unsigned int   host,
 {
     return SOCK_HostPortToString(host, port, buf, buflen);
 }
-
-
-
-/****************************************************************************
- * CRC32
- */
-
-
-/* Standard Ethernet/ZIP polynomial */
-#define CRC32_POLY 0x04C11DB7UL
-
-
-static unsigned int s_CRC32[256];
-
-static void s_CRC32_Init(void)
-{
-    size_t i;
-    if (s_CRC32[255])
-        return;
-    for (i = 0;  i < 256;  i++) {
-        unsigned int byteCRC = (unsigned int) i << 24;
-        int j;
-        for (j = 0;  j < 8;  j++) {
-            if (byteCRC & 0x80000000UL)
-                byteCRC = (byteCRC << 1) ^ CRC32_POLY;
-            else
-                byteCRC = (byteCRC << 1);
-        }
-        s_CRC32[i] = byteCRC;
-    }
-}
-
-
-extern unsigned int CRC32_Update(unsigned int checksum,
-                                 const void *ptr, size_t count)
-{
-    const unsigned char* str = (const unsigned char*) ptr;
-    size_t j;
-
-    s_CRC32_Init();
-    for (j = 0;  j < count;  j++) {
-        size_t i = ((checksum >> 24) ^ *str++) & 0xFF;
-        checksum <<= 8;
-        checksum  ^= s_CRC32[i];
-    }
-    return checksum;
-}
-
-
-
-/****************************************************************************
- * CONNUTIL_GetUsername
- */
-
-
-extern const char* CONNUTIL_GetUsername(char* buf, size_t bufsize)
-{
-#if defined(NCBI_OS_UNIX)
-#  if !defined(NCBI_OS_SOLARIS)  &&  defined(HAVE_GETLOGIN_R)
-#    ifndef LOGIN_NAME_MAX
-#      ifdef _POSIX_LOGIN_NAME_MAX
-#        define LOGIN_NAME_MAX _POSIX_LOGIN_NAME_MAX
-#      else
-#        define LOGIN_NAME_MAX 256
-#      endif
-#    endif
-    char loginbuf[LOGIN_NAME_MAX + 1];
-#  endif
-    struct passwd* pw;
-#  if !defined(NCBI_OS_SOLARIS)  &&  defined(HAVE_GETPWUID_R)
-    struct passwd pwd;
-    char pwdbuf[256];
-#  endif
-#elif defined(NCBI_OS_MSWIN)
-    char  loginbuf[256 + 1];
-    DWORD loginbufsize = sizeof(loginbuf) - 1;
-#endif
-    const char* login;
-
-    assert(buf  &&  bufsize);
-
-#ifndef NCBI_OS_UNIX
-
-#  ifdef NCBI_OS_MSWIN
-    if (GetUserName(loginbuf, &loginbufsize)) {
-        assert(loginbufsize < sizeof(loginbuf));
-        loginbuf[loginbufsize] = '\0';
-        strncpy0(buf, loginbuf, bufsize - 1);
-        return buf;
-    }
-    if ((login = getenv("USERNAME")) != 0) {
-        strncpy0(buf, login, bufsize - 1);
-        return buf;
-    }
-#  endif
-
-#else /*!NCBI_OS_UNIX*/
-
-#  if defined(NCBI_OS_SOLARIS)  ||  !defined(HAVE_GETLOGIN_R)
-    /* NB:  getlogin() is MT-safe on Solaris, yet getlogin_r() comes in two
-     * flavors that differ only in return type, so to make things simpler,
-     * use plain getlogin() here */
-#    ifndef NCBI_OS_SOLARIS
-    CORE_LOCK_WRITE;
-#    endif
-    if ((login = getlogin()) != 0)
-        strncpy0(buf, login, bufsize - 1);
-#    ifndef NCBI_OS_SOLARIS
-    CORE_UNLOCK;
-#    endif
-    if (login)
-        return buf;
-#  else
-    if (getlogin_r(loginbuf, sizeof(loginbuf) - 1) == 0) {
-        loginbuf[sizeof(loginbuf) - 1] = '\0';
-        strncpy0(buf, loginbuf, bufsize - 1);
-        return buf;
-    }
-#  endif
-
-#  if defined(NCBI_OS_SOLARIS)  ||  \
-    (!defined(HAVE_GETPWUID_R)  &&  defined(HAVE_GETPWUID))
-    /* NB:  getpwuid() is MT-safe on Solaris, so use it here, if available */
-#  ifndef NCBI_OS_SOLARIS
-    CORE_LOCK_WRITE;
-#  endif
-    if ((pw = getpwuid(getuid())) != 0  &&  pw->pw_name)
-        strncpy0(buf, pw->pw_name, bufsize - 1);
-#  ifndef NCBI_OS_SOLARIS
-    CORE_UNLOCK;
-#  endif
-    if (pw  &&  pw->pw_name)
-        return buf;
-#  elif defined(HAVE_GETPWUID_R)
-#    if   HAVE_GETPWUID_R == 4
-    /* obsolete but still existent */
-    pw = getpwuid_r(getuid(), &pwd, pwdbuf, sizeof(pwdbuf));
-#    elif HAVE_GETPWUID_R == 5
-    /* POSIX-conforming */
-    if (getpwuid_r(getuid(), &pwd, pwdbuf, sizeof(pwdbuf), &pw) != 0)
-        pw = 0;
-#    else
-#      error "Unknown value of HAVE_GETPWUID_R, 4 or 5 expected."
-#    endif
-    if (pw  &&  pw->pw_name) {
-        assert(pw == &pwd);
-        strncpy0(buf, pw->pw_name, bufsize - 1);
-        return buf;
-    }
-#  endif /*HAVE_GETPWUID_R*/
-
-#endif /*!NCBI_OS_UNIX*/
-
-    /* last resort */
-    if (!(login = getenv("USER"))  &&  !(login = getenv("LOGNAME"))) {
-        buf[0] = '\0';
-        return 0;
-    }
-    strncpy0(buf, login, bufsize - 1);
-    return buf;
-}
-
-
-
-/****************************************************************************
- * Page size granularity
- * See also at corelib's ncbi_system.cpp::GetVirtualMemoryPageSize().
- */
-
-size_t CONNUTIL_GetVMPageSize(void)
-{
-    static size_t ps = 0;
-
-    if (!ps) {
-#if defined(NCBI_OS_MSWIN)
-        SYSTEM_INFO si;
-        GetSystemInfo(&si); 
-        ps = (size_t) si.dwAllocationGranularity;
-#elif defined(NCBI_OS_UNIX) 
-#  if   defined(_SC_PAGESIZE)
-#    define NCBI_SC_PAGESIZE _SC_PAGESIZE
-#  elif defined(_SC_PAGE_SIZE)
-#    define NCBI_SC_PAGESIZE _SC_PAGE_SIZE
-#  elif defined(NCBI_SC_PAGESIZE)
-#    undef  NCBI_SC_PAGESIZE
-#  endif
-#  ifndef   NCBI_SC_PAGESIZE
-        long x = 0;
-#  else
-        long x = sysconf(NCBI_SC_PAGESIZE);
-#    undef  NCBI_SC_PAGESIZE
-#  endif
-        if (x <= 0) {
-#  ifdef HAVE_GETPAGESIZE
-            if ((x = getpagesize()) <= 0)
-                return 0;
-#  else
-            return 0;
-#  endif
-        }
-        ps = (size_t) x;
-#endif /*OS_TYPE*/
-    }
-    return ps;
-}
-
-
-/*
- * --------------------------------------------------------------------------
- * $Log: ncbi_connutil.c,v $
- * Revision 6.108  2006/04/21 14:38:12  lavr
- * ConnNetInfo_GetValue() simplified and sped up considerably
- *
- * Revision 6.107  2006/04/21 01:37:17  lavr
- * SConnNetInfo::lb_disable reinstated along with LB_DISABLE reg/env key
- *
- * Revision 6.105  2006/04/20 13:58:32  lavr
- * Registry keys for new switching scheme for service mappers;
- * Registry keys for LOCAL service mappers;
- * ConnNetInfo_GetValue() exported
- *
- * Revision 6.104  2006/04/19 02:12:06  lavr
- * Call DeleteAllArgs when doing argument overrides
- *
- * Revision 6.103  2006/04/19 01:38:48  lavr
- * ConnNetInfo_{Append|Prepend}Arg sped-up considerably
- *
- * Revision 6.102  2006/04/11 13:51:23  lavr
- * URL_Connect():  Relax on parameter checking
- *
- * Revision 6.101  2006/03/19 01:40:17  lavr
- * Yet another fix for s_IsSufficientAddress()
- *
- * Revision 6.100  2006/03/17 16:42:33  lavr
- * BUF_StripToPattern(): Avoid type-punned ptr (and keep off GCC warning)
- *
- * Revision 6.99  2006/03/16 20:04:02  lavr
- * s_IsSufficientAddress():  IP recognition bug fixed
- *
- * Revision 6.98  2006/03/07 15:37:19  lavr
- * CONNUTIL_GetUsername(): Do not define loginbuf, if it's going to be unused
- *
- * Revision 6.97  2006/02/23 17:42:36  lavr
- * CONNUTIL_GetVMPageSize(): use sysctl() first (fallback to getpagesize())
- *
- * Revision 6.96  2006/02/23 15:23:03  lavr
- * +CONNUTIL_GetVMPageSize() [merely copy of what corelib/ncbi_system.cpp has]
- *
- * Revision 6.95  2006/02/14 15:50:22  lavr
- * Introduce and use CONN_TRACE (via CORE_TRACE) -- NOP in Release mode
- *
- * Revision 6.94  2006/02/01 16:24:24  lavr
- * CONNUTIL_GetUsername(): '\0'-terminate returned result of getlogin_r()
- *
- * Revision 6.93  2006/02/01 00:54:45  ucko
- * One more fix to CONNUTIL_GetUsername: favor LOGIN_NAME_MAX over
- * _POSIX_LOGIN_NAME_MAX, which tends to be smaller, and may not be
- * defined at all on some BSDish systems, and fall back to 256 if neither
- * is defined.
- *
- * Revision 6.92  2006/01/31 20:38:12  lavr
- * CONNUTIL_GetUsername():  Pass pointer to buffer size in GetUserName()
- *
- * Revision 6.91  2006/01/31 20:29:24  lavr
- * CONNUTIL_GetUsername():  use strncpy0 everywhere
- *
- * Revision 6.90  2006/01/31 20:26:46  lavr
- * #include <pwd.h> only us getpwuid[_r]() is known to exist
- *
- * Revision 6.89  2006/01/31 19:32:55  lavr
- * Use GetUserName() as provided by <winbase.h>
- *
- * Revision 6.88  2006/01/31 19:24:40  lavr
- * CONNUTIL_GetUsername():  Clean ASCII username on MS-Windows
- *
- * Revision 6.87  2006/01/31 19:14:28  lavr
- * CONNUTIL_GetUsername():  MS-Windows-specific username discovery
- *
- * Revision 6.86  2006/01/31 17:22:55  lavr
- * CONNUTIL_GetUsername(): Consider USERNAME environment on Windows only
- *
- * Revision 6.85  2006/01/31 17:11:53  lavr
- * MT-safe (as much as possible) implementation of CONNUTIL_GetUsername()
- *
- * Revision 6.84  2006/01/30 18:03:04  lavr
- * Fix CONNUTIL_GetUsername() to return "buf" (instead of constant 0)
- *
- * Revision 6.83  2006/01/27 17:08:56  lavr
- * Spell CONNUTIL_GetUsername() this way
- *
- * Revision 6.82  2006/01/27 17:00:52  lavr
- * New CONNUTIL_GetUsername();  StringHostToPort() and HostPortToString()
- * to remain obsoleted and upcall standardized SOCK_...() replacements
- *
- * Revision 6.81  2006/01/11 20:20:05  lavr
- * -UTIL_ClientAddress()
- * +ConnNetInfo_DeleteAllArgs()
- * +ConnNetInfo_SetupStandardArgs()
- *
- * Revision 6.80  2006/01/11 16:25:02  lavr
- * +UTIL_ClientAddress()
- *
- * Revision 6.79  2005/11/29 21:32:31  lavr
- * Reserve SConnNetInfo::scheme, user, and pass for future use
- *
- * Revision 6.78  2005/11/29 19:54:49  lavr
- * +CRC32 API
- *
- * Revision 6.77  2005/08/18 19:00:13  lavr
- * Fix finishing-up bug in BASE64_Decode()
- *
- * Revision 6.76  2005/06/08 20:42:41  lavr
- * Buffer size better adjustment in ConnNetInfo_AdjustForProxy()
- *
- * Revision 6.75  2005/06/08 16:59:38  lavr
- * ConnNetInfo_ParseURL(): Use default port for absolute URL
- *
- * Revision 6.74  2005/05/13 21:12:23  lavr
- * BASE64_Encode(): fix a critical encoding bug (stray ending char)
- *
- * Revision 6.73  2005/05/02 16:03:24  lavr
- * Better assert() in BASE64_Encode()
- *
- * Revision 6.72  2005/04/20 15:50:30  lavr
- * ConnNetInfo_Create(): Allow zero timeout
- * URL_Connect(): Automagically figure out ANY method based upon body size
- *
- * Revision 6.71  2005/03/21 17:10:03  lavr
- * BASE64_Decode(): unnecessary re-init of "k" removed
- *
- * Revision 6.70  2005/03/21 17:04:35  lavr
- * BASE64_{En|De}code few bugs fixed
- *
- * Revision 6.69  2005/03/19 02:13:55  lavr
- * +BASE64_{En|De}code
- *
- * Revision 6.68  2005/02/28 17:58:31  lavr
- * Cosmetics
- *
- * Revision 6.67  2005/02/24 19:03:11  lavr
- * Always init SConnNetInfo::http_user_header
- *
- * Revision 6.66  2005/02/24 19:00:45  lavr
- * +CONN_HTTP_USER_HEADER
- *
- * Revision 6.65  2004/10/14 13:51:42  lavr
- * StringToHostPort() to allow NULL ptrs
- *
- * Revision 6.64  2004/04/06 19:25:56  lavr
- * Fix ConnNetInfo_DeleteArg() to remove arg's trailing '&'
- *
- * Revision 6.63  2004/01/14 18:52:39  lavr
- * Recognize eMIME_XmlSoap and corresponding text representation "xml+soap"
- *
- * Revision 6.62  2004/01/07 19:23:29  lavr
- * "xml" added as a MIME subtype
- *
- * Revision 6.61  2003/11/12 17:46:12  lavr
- * HostPortToString() changed to be a little more efficient
- *
- * Revision 6.60  2003/08/27 16:27:37  lavr
- * Add "Host:" tag to be able to take advantage of Apache VHosts
- *
- * Revision 6.59  2003/08/25 14:44:43  lavr
- * Employ new k..Timeout constants
- * URL_Connect():  get rid of one avoidable malloc()
- * User header manipulation routines:  factor out some arithmetics for speed
- *
- * Revision 6.58  2003/05/31 05:13:38  lavr
- * Replace bitwise XOR with inequality [of the same effect]
- *
- * Revision 6.57  2003/05/20 21:25:24  lavr
- * Limit SConnNetInfo::max_try by reasonable "short" value
- *
- * Revision 6.56  2003/05/19 16:44:45  lavr
- * Minor style adjustements
- *
- * Revision 6.55  2003/05/14 03:51:54  lavr
- * URL_Connect() rewritten to submit HTTP header as SOCK's initial buffer
- *
- * Revision 6.54  2003/04/30 17:02:11  lavr
- * Name collision resolved in ConnNetInfo_ParseURL()
- *
- * Revision 6.53  2003/03/06 21:55:31  lavr
- * s_ModifyUserHeader(): Heed uninitted usage warning
- * HostPortToString():   Do not append :0 (for zero port) if host is not empty
- *
- * Revision 6.52  2003/02/28 14:47:41  lavr
- * Bugfix: proper bool -> eIO_Status conversion in s_BUF_IO()
- *
- * Revision 6.51  2003/01/31 21:17:04  lavr
- * More robust search for sheme in URL
- *
- * Revision 6.50  2003/01/17 19:44:46  lavr
- * Reduce dependencies
- *
- * Revision 6.49  2003/01/15 19:52:25  lavr
- * *_StripToPattern() calls modified to use Read/PushBack instead of Peek/Read
- *
- * Revision 6.48  2002/12/13 21:19:13  lavr
- * Separate header tag values with spaces as most commonly required (rfc1945)
- *
- * Revision 6.47  2002/12/10 17:34:15  lavr
- * Remove errno decoding on failed connect in URL_Connect()
- *
- * Revision 6.46  2002/12/05 21:43:31  lavr
- * Fix in assignment and compare in URL_Connect()
- *
- * Revision 6.45  2002/12/04 16:50:47  lavr
- * Use SOCK_CreateEx() in URL_Connect()
- *
- * Revision 6.44  2002/11/19 19:19:57  lavr
- * +ConnNetInfo_ExtendUserHeader()
- *
- * Revision 6.43  2002/11/13 19:54:13  lavr
- * ConnNetInfo_DeleteArg(): fix initial argument calculation size
- *
- * Revision 6.42  2002/11/12 05:50:33  lavr
- * Modify client host name discovery
- *
- * Revision 6.41  2002/11/01 20:13:50  lavr
- * Remove MAXHOSTNAMELEN and MAX_IP_ADDR_LEN macros
- *
- * Revision 6.40  2002/10/28 15:42:38  lavr
- * Use "ncbi_ansi_ext.h" privately and use strncpy0()
- *
- * Revision 6.39  2002/10/21 18:30:59  lavr
- * +ConnNetInfo_AppendArg()
- * +ConnNetInfo_PrependArg()
- * +ConnNetInfo_DeleteArg()
- * +ConnNetInfo_PreOverrideArg()
- * +ConnNetInfo_PostOverrideArg()
- *
- * Revision 6.38  2002/10/11 19:42:06  lavr
- * +ConnNetInfo_AppendUserHeader()
- * +ConnNetInfo_OverrideUserHeader()
- * +ConnNetInfo_DeleteUserHeader()
- *
- * Revision 6.37  2002/09/24 18:08:34  lavr
- * Avoid compiler warning in positioning on first but one array element
- *
- * Revision 6.36  2002/08/12 15:12:01  lavr
- * Use persistent SOCK_Write()
- *
- * Revision 6.35  2002/08/07 16:32:47  lavr
- * Changed EIO_ReadMethod enums accordingly; log moved to end
- *
- * Revision 6.34  2002/05/06 19:12:57  lavr
- * -ConnNetInfo_Print(); +ConnNetInfo_Log()
- * Addition: *_StripToPattern() now can strip until EOF (or error)
- *
- * Revision 6.33  2002/04/26 16:31:41  lavr
- * Add more space between functions to separate them better
- *
- * Revision 6.32  2002/04/13 06:36:36  lavr
- * Fix for empty path parsing in HTTP URL
- *
- * Revision 6.31  2002/03/19 22:13:09  lavr
- * Minor tweak in recognizing "infinite" (and part) as a special timeout
- *
- * Revision 6.30  2002/03/11 21:53:18  lavr
- * Recognize ALL in CONN_DEBUG_PRINTOUT; bugfix for '//' in proxy adjustement
- *
- * Revision 6.29  2002/02/20 19:12:17  lavr
- * Swapped eENCOD_Url and eENCOD_None; eENCOD_Unknown introduced
- *
- * Revision 6.28  2002/02/08 22:22:17  lavr
- * BUGFIX: sizeof(info) -> sizeof(*info) in ConnNetInfo_Create()
- *
- * Revision 6.27  2002/01/30 20:14:48  lavr
- * URL_Connect(): Print error code in some failure messages
- *
- * Revision 6.26  2002/01/28 20:21:46  lavr
- * Do not store "" as a user_header
- *
- * Revision 6.25  2001/12/30 19:40:32  lavr
- * +ConnNetInfo_ParseURL()
- * Added recordkeeping of service name for which the info was created
- *
- * Revision 6.24  2001/12/04 15:56:28  lavr
- * Use strdup() instead of explicit strcpy(malloc(...), ...)
- *
- * Revision 6.23  2001/09/24 20:27:00  lavr
- * Message corrected: "Adjusted path too long"
- *
- * Revision 6.22  2001/09/10 21:14:58  lavr
- * Added functions: StringToHostPort()
- *                  HostPortToString()
- *
- * Revision 6.21  2001/05/31 21:30:57  vakatov
- * MIME_ParseContentTypeEx() -- a more accurate parsing
- *
- * Revision 6.20  2001/05/29 21:15:43  vakatov
- * + eMIME_Plain
- *
- * Revision 6.19  2001/04/24 21:29:43  lavr
- * Special text value "infinite" accepted as infinite timeout from environment
- *
- * Revision 6.18  2001/03/26 18:37:09  lavr
- * #include <ctype.h> not used, removed
- *
- * Revision 6.17  2001/03/02 20:08:05  lavr
- * Typo fixed
- *
- * Revision 6.16  2001/01/25 16:58:33  lavr
- * ConnNetInfo_SetUserHeader now used throughout to set/reset http_user_header
- *
- * Revision 6.15  2001/01/23 23:06:18  lavr
- * SConnNetInfo.debug_printout converted from boolean to enum
- * BUF_StripToPattern() introduced
- *
- * Revision 6.14  2001/01/12 00:01:27  lavr
- * CONN_PROXY_HOST was forgotten to init in ConnNetInfo_Create
- *
- * Revision 6.13  2001/01/11 23:07:08  lavr
- * ConnNetInfo_Print() prints user-header 'as is'; pretty-printing undone
- *
- * Revision 6.12  2001/01/08 23:46:27  lavr
- * REQUEST_METHOD -> REQ_METHOD to be consistent with SConnNetInfo
- *
- * Revision 6.11  2001/01/08 22:35:56  lavr
- * Client-Mode removed; replaced by 2 separate boolean fields:
- * stateless and firewall
- *
- * Revision 6.10  2000/12/29 17:54:11  lavr
- * NCBID stuff removed; ConnNetInfo_SetUserHeader added;
- * modifications to ConnNetInfo_Print output.
- *
- * Revision 6.9  2000/11/07 23:23:18  vakatov
- * In-sync with the C Toolkit "connutil.c:R6.15", "connutil.h:R6.13"
- * (with "eMIME_Dispatch" added).
- *
- * Revision 6.8  2000/10/20 17:08:40  lavr
- * All keys capitalized for registry access
- * Search for some keywords made case insensitive
- *
- * Revision 6.7  2000/10/11 22:29:44  lavr
- * Forgotten blank added after {GET|POST} request in URL_Connect
- *
- * Revision 6.6  2000/10/05 22:35:24  lavr
- * SConnNetInfo modified to contain 'client_mode' instead of just
- * 'firewall' boolean
- *
- * Revision 6.5  2000/09/27 19:37:40  lavr
- * ncbi_ansi_ext.h included
- *
- * Revision 6.4  2000/09/26 22:01:33  lavr
- * Registry entries changed, HTTP request method added
- *
- * Revision 6.3  2000/04/21 19:42:35  vakatov
- * Several minor typo/bugs fixed
- *
- * Revision 6.2  2000/03/29 16:36:09  vakatov
- * MIME_ParseContentType() -- a fix
- *
- * Revision 6.1  2000/03/24 22:53:34  vakatov
- * Initial revision
- *
- * ==========================================================================
- */
