@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_CORE__H
 #define CONNECT___NCBI_CORE__H
 
-/* $Id: ncbi_core.h,v 6.30 2006/07/27 17:00:39 lavr Exp $
+/* $Id: ncbi_core.h,v 6.31 2007/06/25 15:25:32 kazimird Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -168,9 +168,11 @@ typedef struct MT_LOCK_tag* MT_LOCK;
  *
  */
 typedef enum {
-    eMT_Lock,      /**< lock   critical section             */
-    eMT_LockRead,  /**< lock   critical section for reading */
-    eMT_Unlock     /**< unlock critical section             */
+    eMT_Lock,        /**< lock   critical section                     */
+    eMT_LockRead,    /**< lock   critical section for reading         */
+    eMT_Unlock,      /**< unlock critical section                     */
+    eMT_TryLock,     /**< try to lock,             return immediately */
+    eMT_TryLockRead  /**< try to lock for reading, return immediately */
 } EMT_Lock;
 
 
@@ -685,105 +687,4 @@ extern NCBI_XCONNECT_EXPORT void REG_Set
 
 /* @} */
 
-
-/*
- * ===========================================================================
- *
- * $Log: ncbi_core.h,v $
- * Revision 6.30  2006/07/27 17:00:39  lavr
- * Corrected doxygenization of NOTEs
- *
- * Revision 6.29  2006/07/27 15:42:50  lavr
- * More doxygenization
- *
- * Revision 6.28  2006/07/13 21:00:22  lavr
- * Doxygenization
- *
- * Revision 6.27  2005/04/20 18:10:53  lavr
- * verify() moved away into a private header (ncbi_assert.h)
- *
- * Revision 6.26  2004/12/27 15:30:08  lavr
- * +eIO_WriteOutOfBand
- *
- * Revision 6.25  2004/03/12 23:26:22  gorelenk
- * Added export prefixes.
- *
- * Revision 6.24  2003/09/02 20:45:45  lavr
- * -<connect/connect_export.h> -- now included from <connect/ncbi_types.h>
- *
- * Revision 6.23  2003/04/09 17:58:48  siyan
- * Added doxygen support
- *
- * Revision 6.22  2003/03/06 19:42:47  lavr
- * Isolate special inclusion of <stdio.h>, <stdlib.h> in DEBUG branch only
- *
- * Revision 6.21  2003/03/06 01:42:14  lavr
- * Include <stdio.h> and <stdlib.h> required by <assert.h> under Codewarrior
- *
- * Revision 6.20  2003/01/08 01:59:32  lavr
- * DLL-ize CONNECT library for MSVC (add NCBI_XCONNECT_EXPORT)
- *
- * Revision 6.19  2002/09/19 18:00:33  lavr
- * Header file guard macro changed
- *
- * Revision 6.18  2002/08/13 19:26:39  lavr
- * Add eIO_Interrupt to EIO_Status
- *
- * Revision 6.17  2002/08/08 14:21:11  ucko
- * Fix C++-style comment; really move CVS log to end.
- *
- * Revision 6.16  2002/08/07 16:28:25  lavr
- * Added EIO_WriteMethod; EIO_ReadMethod enums changed; log moved to end
- *
- * Revision 6.15  2002/05/06 19:07:48  lavr
- * Added notes to EIO_Event enum type
- *
- * Revision 6.14  2002/03/28 13:28:43  kans
- * undef verify if already defined, such as on Mac OS X Darwin (EN)
- *
- * Revision 6.13  2001/09/27 22:29:35  vakatov
- * Always define "NDEBUG" if "_DEBUG" is not defined (mostly for the C Toolkit)
- *
- * Revision 6.12  2001/08/09 16:22:51  lavr
- * Remove last (unneeded) parameter from LOG_Reset()
- *
- * Revision 6.11  2001/06/19 20:16:18  lavr
- * Added #include <connect/ncbi_types.h>
- *
- * Revision 6.10  2001/06/19 19:08:28  lavr
- * STimeout and ESwitch moved to <connect/ncbi_types.h>
- *
- * Revision 6.9  2001/05/17 18:10:22  vakatov
- * Moved the logging macros from <ncbi_core.h> to <ncbi_util.h>.
- * Logging::  always call the logger if severity is eLOG_Fatal.
- *
- * Revision 6.8  2001/03/02 20:06:54  lavr
- * Typos fixed
- *
- * Revision 6.7  2001/01/23 23:07:30  lavr
- * A make-up change
- *
- * Revision 6.6  2001/01/11 16:41:18  lavr
- * Registry Get/Set methods got the 'user_data' argument, forgotten earlier
- *
- * Revision 6.5  2000/10/18 20:29:41  vakatov
- * REG_Get::  pass in the default value (rather than '\0')
- *
- * Revision 6.4  2000/06/23 19:34:41  vakatov
- * Added means to log binary data
- *
- * Revision 6.3  2000/04/07 19:55:14  vakatov
- * Standard indentation
- *
- * Revision 6.2  2000/03/24 23:12:03  vakatov
- * Starting the development quasi-branch to implement CONN API.
- * All development is performed in the NCBI C++ tree only, while
- * the NCBI C tree still contains "frozen" (see the last revision) code.
- *
- * Revision 6.1  2000/02/23 22:30:40  vakatov
- * Initial revision
- *
- * ===========================================================================
- */
-
 #endif /* CONNECT___NCBI_CORE__H */
