@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_SERVICE__H
 #define CONNECT___NCBI_SERVICE__H
 
-/*  $Id: ncbi_service.h,v 6.49 2006/07/06 18:48:40 lavr Exp $
+/*  $Id: ncbi_service.h,v 6.52 2007/05/09 13:55:29 kazimird Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -37,9 +37,9 @@
 #include <connect/ncbi_host_info.h>
 
 
-/* Revision 6.220 */
+/* Revision 6.230 */
 #define SERV_CLIENT_REVISION_MAJOR  6
-#define SERV_CLIENT_REVISION_MINOR  220
+#define SERV_CLIENT_REVISION_MINOR  230
 
 
 /** @addtogroup ServiceSupport
@@ -93,16 +93,22 @@ extern NCBI_XCONNECT_EXPORT SERV_ITER SERV_OpenSimple
 #define SERV_LOCALHOST ((unsigned int)(~0UL))
 #define SERV_ANYHOST   0             /* default, may be used as just 0       */
 
-/* Special "type" bit values that may be combined with server types */
+/* Special "type" bit values that may be combined with server types.
+ * NB:  Also, MSBs should be kept compatible with EMGHBN_Option */
 typedef enum {
-    /* Allows to get even dead services (but not off ones!) */
-    fSERV_Promiscuous = 0x40000000,  /* NB: all preference params are ignored*/
-    /* Do reverse DNS translation of the would-be resulting info */
-    fSERV_ReverseDns  = 0x20000000,
+    fSERV_Any               = 0,
+    fSERV_All               = 0x0000FFFF,
     /* Only stateless servers should be returned */
-    fSERV_Stateless   = 0x10000000,
-    fSERV_All         = 0x0000FFFF,
-    fSERV_Any         = 0
+    fSERV_Stateless         = 0x00100000,
+    fSERV_Reserved_1        = 0x00200000, /*MBZ*/
+    /* Do reverse DNS translation of the would-be resulting info */
+    fSERV_ReverseDns        = 0x00800000,
+    fSERV_Reserved_2        = 0x10000000, /*MBZ*/
+    /* Allows to get even down services (but not the off ones!)
+     * NB: flex preference params are ignored */
+    fSERV_IncludeDown       = 0x20000000,
+    fSERV_IncludeSuppressed = 0x40000000,
+    fSERV_Promiscuous       = 0x60000000
 } ESERV_SpecialType;
 typedef unsigned int TSERV_Type;     /* Bitwise OR of ESERV_[Special]Type    */
 
@@ -216,160 +222,4 @@ extern NCBI_XCONNECT_EXPORT void SERV_Close
 
 /* @} */
 
-
-/*
- * --------------------------------------------------------------------------
- * $Log: ncbi_service.h,v $
- * Revision 6.49  2006/07/06 18:48:40  lavr
- * Some fixes in comments
- *
- * Revision 6.48  2006/06/07 20:23:07  lavr
- * Bump up minor client version number to 220 (full now is 6.220)
- *
- * Revision 6.47  2006/04/20 19:27:18  lavr
- * More comments for SERV_Open*() family of calls
- *
- * Revision 6.46  2006/03/06 20:24:44  lavr
- * Comments
- *
- * Revision 6.45  2006/01/03 19:57:59  lavr
- * Bump client revision to 6.210
- *
- * Revision 6.44  2005/12/23 18:07:34  lavr
- * Special service flags moved here, values reassigned
- *
- * Revision 6.43  2005/12/14 22:03:53  lavr
- * ESERV_SpecialType reinstated public
- *
- * Revision 6.42  2005/12/14 21:17:09  lavr
- * ESERV_SpecialType made private (moved to ncbi_servicep.h in source tree)
- * SSERV_InfoCPtr introduced;  some API usage comments extended/corrected
- *
- * Revision 6.41  2005/10/03 15:55:22  lavr
- * Change ESERV_SpecialType::fSERV_Promiscuous to fit "signed int"
- *
- * Revision 6.40  2005/09/02 14:28:18  lavr
- * Bump client revision number to 6.200 for use with patched PubFetch
- *
- * Revision 6.39  2005/07/06 18:54:44  lavr
- * +enum ESERV_SpecialType to hold special server type bits (instead of macros)
- *
- * Revision 6.38  2005/07/06 18:27:58  lavr
- * Eliminate macro calls
- *
- * Revision 6.37  2005/05/04 16:13:57  lavr
- * DISP_SetMessageHook() moved to <connect/ncbi_service_misc.h>
- *
- * Revision 6.36  2005/03/05 21:04:11  lavr
- * +fSERV_ReverseDns
- *
- * Revision 6.35  2005/01/31 17:08:35  lavr
- * info -> net_info where appropriate for consistency
- *
- * Revision 6.34  2004/08/19 15:26:54  lavr
- * +fSERV_Promiscuous
- *
- * Revision 6.33  2004/06/14 16:36:13  lavr
- * Client minor version number incremented
- *
- * Revision 6.32  2004/01/30 14:37:33  lavr
- * Client revision made independent of CVS revisions
- *
- * Revision 6.31  2003/08/11 19:05:54  lavr
- * +DISP_SetMessageHook()
- *
- * Revision 6.30  2003/06/12 13:20:59  lavr
- * Added notes for SERV_GetNextInfoEx()
- *
- * Revision 6.29  2003/04/09 19:05:50  siyan
- * Added doxygen support
- *
- * Revision 6.28  2003/01/08 01:59:33  lavr
- * DLL-ize CONNECT library for MSVC (add NCBI_XCONNECT_EXPORT)
- *
- * Revision 6.27  2002/10/28 20:12:20  lavr
- * +ncbi_host_info.h
- *
- * Revision 6.26  2002/09/19 18:05:00  lavr
- * Header file guard macro changed; log moved to end
- *
- * Revision 6.25  2002/05/06 19:08:35  lavr
- * Removed mention of IP substitutions (mistakenly left behind a while ago)
- *
- * Revision 6.24  2002/04/13 06:35:10  lavr
- * Fast track routine SERV_GetInfoEx(), many syscalls optimizations
- *
- * Revision 6.23  2001/09/26 16:55:10  lavr
- * Revision log updated; in-line documentation corrected
- *
- * Revision 6.22  2001/09/24 20:22:59  lavr
- * Added function: SERV_Reset()
- *
- * Revision 6.21  2001/09/10 21:18:35  lavr
- * SERV_GetNextInfoEx(): Description of IP address subst in host environment
- *
- * Revision 6.20  2001/08/20 21:57:49  lavr
- * Parameter change for clarity: info -> net_info if type is SConnNetInfo
- *
- * Revision 6.19  2001/07/18 17:39:56  lavr
- * Comment added for SERV_OpenSimple() to note about absence of preferred host
- *
- * Revision 6.18  2001/06/25 15:32:06  lavr
- * Added function: SERV_GetNextInfoEx()
- * Both SERV_Open() and SERV_GetNextInfo() made macros for faster access
- *
- * Revision 6.17  2001/06/11 22:14:44  lavr
- * Include files adjusted
- *
- * Revision 6.16  2001/06/04 17:00:10  lavr
- * Include files adjusted
- *
- * Revision 6.15  2001/04/26 14:18:20  lavr
- * SERV_MapperName() moved to a private header
- *
- * Revision 6.14  2001/04/24 21:15:35  lavr
- * Added functions: SERV_MapperName(), SERV_Penalize()
- *
- * Revision 6.13  2001/03/02 20:05:56  lavr
- * SERV_LOCALHOST added; SERV_OpenSimple() made more documented
- *
- * Revision 6.12  2001/02/09 17:32:52  lavr
- * Modified: fSERV_StatelessOnly overrides info->stateless
- *
- * Revision 6.11  2001/01/08 22:48:00  lavr
- * Double return 0 in GetNextInfo() removed:
- * 0 now indicates an error unconditionally
- *
- * Revision 6.10  2000/12/29 17:40:30  lavr
- * Pretty printed; Double 0 return added to SERV_GetNextInfo()
- *
- * Revision 6.9  2000/12/06 22:17:02  lavr
- * Binary host addresses are now explicitly stated to be in network byte
- * order, whereas binary port addresses now use native (host) representation
- *
- * Revision 6.8  2000/10/20 17:03:49  lavr
- * Added 'const' to SConnNetInfo in SERV_OpenEx()
- *
- * Revision 6.7  2000/10/05 22:40:06  lavr
- * Additional parameter 'info' in a call to SERV_Open()
- *
- * Revision 6.6  2000/10/05 21:26:30  lavr
- * Log message edited
- *
- * Revision 6.5  2000/10/05 21:10:11  lavr
- * Parameters to SERV_Open() and SERV_OpenEx() changed
- *
- * Revision 6.4  2000/05/31 23:12:17  lavr
- * First try to assemble things together to get working service mapper
- *
- * Revision 6.3  2000/05/22 16:53:07  lavr
- * Rename service_info -> server_info everywhere
- * (including file names) as the latter name is more relevant
- *
- * Revision 6.2  2000/05/12 18:29:22  lavr
- * First working revision
- *
- * ==========================================================================
- */
-
 #endif /* CONNECT___NCBI_SERVICE__H */
