@@ -1,4 +1,4 @@
-/* $Id: ncbi_util.c,v 6.37 2006/07/26 14:38:07 lavr Exp $
+/* $Id: ncbi_util.c,v 6.42 2007/07/31 16:25:29 kazimird Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -58,6 +58,8 @@
 #  include <windows.h>
 #endif
 
+#define NCBI_USE_PRECOMPILED_CRC32_TABLES 1
+
 
 /* Static function pre-declarations to avoid C++ compiler warnings
  */
@@ -111,17 +113,29 @@ extern LOG CORE_GetLOG(void)
 }
 
 
-extern void CORE_SetLOGFILE
+extern void CORE_SetLOGFILE_Ex
 (FILE*       fp,
- int/*bool*/ auto_close)
+ ELOG_Level  cut_off,
+ int/*bool*/ auto_close
+ )
 {
     LOG lg = LOG_Create(0, 0, 0, 0);
-    LOG_ToFILE(lg, fp, auto_close);
+    LOG_ToFILE_Ex(lg, fp, cut_off, auto_close);
     CORE_SetLOG(lg);
 }
 
 
-extern int/*bool*/ CORE_SetLOGFILE_NAME(const char* filename)
+extern void CORE_SetLOGFILE
+(FILE*       fp,
+ int/*bool*/ auto_close)
+{
+    CORE_SetLOGFILE_Ex(fp, 0, auto_close);
+}
+
+
+extern int/*bool*/ CORE_SetLOGFILE_NAME_Ex
+(const char* filename,
+ ELOG_Level  cut_off)
 {
     FILE* fp = fopen(filename, "a");
     if ( !fp ) {
@@ -129,11 +143,19 @@ extern int/*bool*/ CORE_SetLOGFILE_NAME(const char* filename)
         return 0/*false*/;
     }
 
-    CORE_SetLOGFILE(fp, 1/*true*/);
+    CORE_SetLOGFILE_Ex(fp, cut_off, 1/*true*/);
     return 1/*true*/;
 }
 
 
+extern int/*bool*/ CORE_SetLOGFILE_NAME
+(const char* filename
+ )
+{
+    return CORE_SetLOGFILE_NAME_Ex(filename, 0);
+}
+
+
 static TLOG_FormatFlags s_LogFormatFlags = fLOG_Default;
 
 extern TLOG_FormatFlags CORE_SetLOGFormatFlags(TLOG_FormatFlags flags)
@@ -145,16 +167,29 @@ extern TLOG_FormatFlags CORE_SetLOGFormatFlags(TLOG_FormatFlags flags)
 }
 
 
+#ifdef __GNUC__
+inline
+#endif /*__GNUC__*/
+static int/*bool*/ s_IsQuoted(unsigned char c)
+{
+    return (c == '\t'  ||   c == '\v'  ||  c == '\b'  ||
+            c == '\r'  ||   c == '\f'  ||  c == '\a'  ||
+            c == '\n'  ||   c == '\\'  ||  c == '\''  ||
+            c == '"' ? 1/*true*/ : 0/*false*/);
+}
+
+
 extern char* LOG_ComposeMessage
 (const SLOG_Handler* call_data,
  TLOG_FormatFlags    format_flags)
 {
-    static const char s_RawData_Begin[] =
+    static const char kRawData_Begin[] =
         "\n#################### [BEGIN] Raw Data (%lu byte%s):\n";
-    static const char s_RawData_End[] =
+    static const char kRawData_End[] =
         "\n#################### [END] Raw Data\n";
 
-    char* str, datetime[32];
+    char *str, *s, datetime[32];
+    const char* level = 0;
 
     /* Calculated length of ... */
     size_t datetime_len  = 0;
@@ -210,10 +245,11 @@ extern char* LOG_ComposeMessage
         datetime_len = strftime(datetime, sizeof(datetime), timefmt, tm);
 #endif/*NCBI_OS_MSWIN*/
     }
-    if ((format_flags & fLOG_Level) != 0  &&
-        (call_data->level != eLOG_Note ||
-         !(format_flags & fLOG_OmitNoteLevel))) {
-        level_len = strlen(LOG_LevelStr(call_data->level)) + 2;
+    if ((format_flags & fLOG_Level) != 0
+        &&  (call_data->level != eLOG_Note
+             ||  !(format_flags & fLOG_OmitNoteLevel))) {
+        level = LOG_LevelStr(call_data->level);
+        level_len = strlen(level) + 2;
     }
     if ((format_flags & fLOG_Module) != 0  &&
         call_data->module  &&  *call_data->module) {
@@ -229,131 +265,191 @@ extern char* LOG_ComposeMessage
 
     if ( call_data->raw_size ) {
         const unsigned char* d = (const unsigned char*) call_data->raw_data;
-        size_t      i = call_data->raw_size;
+        size_t i = call_data->raw_size;
         for (data_len = 0;  i;  i--, d++) {
-            if (*d == '\\'  ||  *d == '\r'  ||  *d == '\t') {
+            if (s_IsQuoted(*d)) {
                 data_len++;
-            } else if (!isprint(*d)  &&  *d != '\n') {
-                data_len += 2;
+            } else if (!isprint(*d)) {
+                data_len += 3;
             }
         }
-        data_len += sizeof(s_RawData_Begin) + 20 + call_data->raw_size +
-            sizeof(s_RawData_End);
+        data_len += (sizeof(kRawData_Begin) + 20 + call_data->raw_size +
+                     sizeof(kRawData_End));
     }
 
     /* Allocate memory for the resulting message */
-    total_len = datetime_len + file_line_len + module_len + level_len +
-        message_len + data_len;
-    str = (char*) malloc(total_len + 1);
-    if ( !str ) {
+    total_len = (datetime_len + file_line_len + module_len
+                 + level_len + message_len + data_len);
+    if (!(str = (char*) malloc(total_len + 1))) {
         assert(0);
         return 0;
     }
 
+    s = str;
     /* Compose the message */
-    str[0] = '\0';
     if ( datetime_len ) {
-        strcpy(str, datetime);
+        memcpy(s, datetime, datetime_len);
+        s += datetime_len;
     }
     if ( file_line_len ) {
-        sprintf(str + strlen(str), "\"%s\", line %d: ",
-                call_data->file, (int) call_data->line);
+        s += sprintf(s, "\"%s\", line %d: ",
+                     call_data->file, (int) call_data->line);
     }
     if ( module_len ) {
-        strcat(str, "[");
-        strcat(str, call_data->module);
-        strcat(str, "] ");
+        *s++ = '[';
+        memcpy(s, call_data->module, module_len -= 3);
+        s += module_len;
+        *s++ = ']';
+        *s++ = ' ';
     }
     if ( level_len ) {
-        strcat(str, LOG_LevelStr(call_data->level));
-        strcat(str, ": ");
+        memcpy(s, level, level_len -= 2);
+        s += level_len;
+        *s++ = ':';
+        *s++ = ' ';
     }
     if ( message_len ) {
-        strcat(str, call_data->message);
+        memcpy(s, call_data->message, message_len);
+        s += message_len;
     }
     if ( data_len ) {
-        size_t i;
-        char* s;
         const unsigned char* d;
+        size_t i;
 
-        s = str + strlen(str);
-        sprintf(s, s_RawData_Begin, (unsigned long) call_data->raw_size,
-                call_data->raw_size == 1 ? "" : "s");
-        s += strlen(s);
+        s += sprintf(s, kRawData_Begin, (unsigned long) call_data->raw_size,
+                     &"s"[call_data->raw_size == 1]);
 
         d = (const unsigned char*) call_data->raw_data;
         for (i = call_data->raw_size;  i;  i--, d++) {
-            if (*d == '\\') {
-                *s++ = '\\';
-                *s++ = '\\';
-            } else if (*d == '\r') {
-                *s++ = '\\';
-                *s++ = 'r';
-            } else if (*d == '\t') {
+            switch (*d) {
+            case '\t':
                 *s++ = '\\';
                 *s++ = 't';
-            } else if (!isprint(*d)  &&  *d != '\n') {
-                static const char s_Hex[16] = {
-                    '0', '1', '2', '3', '4', '5', '6', '7',
-                    '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'
-                };
+                continue;
+            case '\v':
                 *s++ = '\\';
-                *s++ = s_Hex[*d / 16];
-                *s++ = s_Hex[*d % 16];
-            } else {
-                *s++ = (char) *d;
+                *s++ = 'v';
+                continue;
+            case '\b':
+                *s++ = '\\';
+                *s++ = 'b';
+                continue;
+            case '\r':
+                *s++ = '\\';
+                *s++ = 'r';
+                continue;
+            case '\f':
+                *s++ = '\\';
+                *s++ = 'f';
+                continue;
+            case '\a':
+                *s++ = '\\';
+                *s++ = 'a';
+                continue;
+            case '\n':
+            case '\\':
+            case '\'':
+            case '"':
+                *s++ = '\\';
+                break;
+            default:
+                if (!isprint(*d)) {
+                    int/*bool*/ reduce;
+                    unsigned char v;
+                    reduce = (i == 1          ||  s_IsQuoted(d[1])  ||
+                              !isprint(d[1])  ||  d[1] < '0'  ||  d[1] > '7');
+                    *s++ = '\\';
+                    v =  *d >> 6;
+                    if (v  ||  !reduce) {
+                        *s++ = '0' + v;
+                        reduce = 0;
+                    }
+                    v = (*d >> 3) & 7;
+                    if (v  ||  !reduce)
+                        *s++ = '0' + v;
+                    v =  *d & 7;
+                    *s++ =     '0' + v;
+                    continue;
+                }
+                break;
             }
+            *s++ = (char) *d;
         }
 
-        strcpy(s, s_RawData_End);
-    }
+        memcpy(s, kRawData_End, sizeof(kRawData_End));
+    } else
+        *s = '\0';
 
     assert(strlen(str) <= total_len);
     return str;
 }
 
 
-/* Callback for LOG_Reset_FILE() */
+typedef struct {
+    FILE*       fp;
+    int/*bool*/ cut_off;
+    int/*bool*/ auto_close;
+} SLogData;
+
+
+/* Callback for LOG_ToFILE[_Ex]() */
 static void s_LOG_FileHandler(void* user_data, SLOG_Handler* call_data)
 {
-    FILE* fp = (FILE*) user_data;
+    SLogData* data = (SLogData*) user_data;
+    assert(data  &&  data->fp);
     assert(call_data);
 
-    if ( fp ) {
+    if (call_data->level >= data->cut_off  ||  call_data->level == eLOG_Fatal){
         char* str = LOG_ComposeMessage(call_data, s_LogFormatFlags);
         if ( str ) {
-            fprintf(fp, "%s\n", str);
-            fflush(fp);
+            fprintf(data->fp, "%s\n", str);
+            fflush(data->fp);
             free(str);
         }
     }
 }
 
 
-/* Callback for LOG_Reset_FILE() */
+/* Callback for LOG_ToFILE[_Ex]() */
 static void s_LOG_FileCleanup(void* user_data)
 {
-    FILE* fp = (FILE*) user_data;
+    SLogData* data = (SLogData*) user_data;
 
-    if ( fp )
-        fclose(fp);
+    assert(data  &&  data->fp);
+    if (data->auto_close)
+        fclose(data->fp);
+    else
+        fflush(data->fp);
+    free(user_data);
+}
+
+
+extern void LOG_ToFILE_Ex
+(LOG         lg,
+ FILE*       fp,
+ ELOG_Level  cut_off,
+ int/*bool*/ auto_close
+ )
+{
+    SLogData* data = (SLogData*)(fp ? malloc(sizeof(*data)) : 0);
+    if ( data ) {
+        data->fp         = fp;
+        data->cut_off    = cut_off;
+        data->auto_close = auto_close;
+        LOG_Reset(lg, data, s_LOG_FileHandler, s_LOG_FileCleanup);
+    } else {
+        LOG_Reset(lg, 0/*data*/, 0/*handler*/, 0/*cleanup*/);
+    }
 }
 
 
 extern void LOG_ToFILE
 (LOG         lg,
  FILE*       fp,
- int/*bool*/ auto_close)
+ int/*bool*/ auto_close
+ )
 {
-    if ( fp ) {
-        if ( auto_close ) {
-            LOG_Reset(lg, fp, s_LOG_FileHandler, s_LOG_FileCleanup);
-        } else {
-            LOG_Reset(lg, fp, s_LOG_FileHandler, 0/*no cleaning up*/);
-        }
-    } else {
-        LOG_Reset(lg, 0/*data*/, 0/*handler*/, 0/*cleanup*/);
-    }
+    LOG_ToFILE_Ex(lg, fp, 0, auto_close);
 }
 
 
@@ -450,9 +546,8 @@ extern char* MessagePlusErrno
 
         /* print error code */
         for ( ;  mod;  mod /= 10) {
-            static const char s_Num[] = "0123456789";
             assert(x_errno / mod < 10);
-            *beg++ = s_Num[x_errno / mod];
+            *beg++ = '0' + x_errno / mod;
             x_errno %= mod;
         }
         /* "," before "<descr>" */
@@ -677,40 +772,119 @@ size_t CORE_GetVMPageSize(void)
 #define CRC32_POLY 0x04C11DB7UL
 
 
-static unsigned int s_CRC32[256];
+#ifdef NCBI_USE_PRECOMPILED_CRC32_TABLES
 
+static const unsigned int s_CRC32[256] = {
+    0x00000000, 0x04c11db7, 0x09823b6e, 0x0d4326d9,
+    0x130476dc, 0x17c56b6b, 0x1a864db2, 0x1e475005,
+    0x2608edb8, 0x22c9f00f, 0x2f8ad6d6, 0x2b4bcb61,
+    0x350c9b64, 0x31cd86d3, 0x3c8ea00a, 0x384fbdbd,
+    0x4c11db70, 0x48d0c6c7, 0x4593e01e, 0x4152fda9,
+    0x5f15adac, 0x5bd4b01b, 0x569796c2, 0x52568b75,
+    0x6a1936c8, 0x6ed82b7f, 0x639b0da6, 0x675a1011,
+    0x791d4014, 0x7ddc5da3, 0x709f7b7a, 0x745e66cd,
+    0x9823b6e0, 0x9ce2ab57, 0x91a18d8e, 0x95609039,
+    0x8b27c03c, 0x8fe6dd8b, 0x82a5fb52, 0x8664e6e5,
+    0xbe2b5b58, 0xbaea46ef, 0xb7a96036, 0xb3687d81,
+    0xad2f2d84, 0xa9ee3033, 0xa4ad16ea, 0xa06c0b5d,
+    0xd4326d90, 0xd0f37027, 0xddb056fe, 0xd9714b49,
+    0xc7361b4c, 0xc3f706fb, 0xceb42022, 0xca753d95,
+    0xf23a8028, 0xf6fb9d9f, 0xfbb8bb46, 0xff79a6f1,
+    0xe13ef6f4, 0xe5ffeb43, 0xe8bccd9a, 0xec7dd02d,
+    0x34867077, 0x30476dc0, 0x3d044b19, 0x39c556ae,
+    0x278206ab, 0x23431b1c, 0x2e003dc5, 0x2ac12072,
+    0x128e9dcf, 0x164f8078, 0x1b0ca6a1, 0x1fcdbb16,
+    0x018aeb13, 0x054bf6a4, 0x0808d07d, 0x0cc9cdca,
+    0x7897ab07, 0x7c56b6b0, 0x71159069, 0x75d48dde,
+    0x6b93dddb, 0x6f52c06c, 0x6211e6b5, 0x66d0fb02,
+    0x5e9f46bf, 0x5a5e5b08, 0x571d7dd1, 0x53dc6066,
+    0x4d9b3063, 0x495a2dd4, 0x44190b0d, 0x40d816ba,
+    0xaca5c697, 0xa864db20, 0xa527fdf9, 0xa1e6e04e,
+    0xbfa1b04b, 0xbb60adfc, 0xb6238b25, 0xb2e29692,
+    0x8aad2b2f, 0x8e6c3698, 0x832f1041, 0x87ee0df6,
+    0x99a95df3, 0x9d684044, 0x902b669d, 0x94ea7b2a,
+    0xe0b41de7, 0xe4750050, 0xe9362689, 0xedf73b3e,
+    0xf3b06b3b, 0xf771768c, 0xfa325055, 0xfef34de2,
+    0xc6bcf05f, 0xc27dede8, 0xcf3ecb31, 0xcbffd686,
+    0xd5b88683, 0xd1799b34, 0xdc3abded, 0xd8fba05a,
+    0x690ce0ee, 0x6dcdfd59, 0x608edb80, 0x644fc637,
+    0x7a089632, 0x7ec98b85, 0x738aad5c, 0x774bb0eb,
+    0x4f040d56, 0x4bc510e1, 0x46863638, 0x42472b8f,
+    0x5c007b8a, 0x58c1663d, 0x558240e4, 0x51435d53,
+    0x251d3b9e, 0x21dc2629, 0x2c9f00f0, 0x285e1d47,
+    0x36194d42, 0x32d850f5, 0x3f9b762c, 0x3b5a6b9b,
+    0x0315d626, 0x07d4cb91, 0x0a97ed48, 0x0e56f0ff,
+    0x1011a0fa, 0x14d0bd4d, 0x19939b94, 0x1d528623,
+    0xf12f560e, 0xf5ee4bb9, 0xf8ad6d60, 0xfc6c70d7,
+    0xe22b20d2, 0xe6ea3d65, 0xeba91bbc, 0xef68060b,
+    0xd727bbb6, 0xd3e6a601, 0xdea580d8, 0xda649d6f,
+    0xc423cd6a, 0xc0e2d0dd, 0xcda1f604, 0xc960ebb3,
+    0xbd3e8d7e, 0xb9ff90c9, 0xb4bcb610, 0xb07daba7,
+    0xae3afba2, 0xaafbe615, 0xa7b8c0cc, 0xa379dd7b,
+    0x9b3660c6, 0x9ff77d71, 0x92b45ba8, 0x9675461f,
+    0x8832161a, 0x8cf30bad, 0x81b02d74, 0x857130c3,
+    0x5d8a9099, 0x594b8d2e, 0x5408abf7, 0x50c9b640,
+    0x4e8ee645, 0x4a4ffbf2, 0x470cdd2b, 0x43cdc09c,
+    0x7b827d21, 0x7f436096, 0x7200464f, 0x76c15bf8,
+    0x68860bfd, 0x6c47164a, 0x61043093, 0x65c52d24,
+    0x119b4be9, 0x155a565e, 0x18197087, 0x1cd86d30,
+    0x029f3d35, 0x065e2082, 0x0b1d065b, 0x0fdc1bec,
+    0x3793a651, 0x3352bbe6, 0x3e119d3f, 0x3ad08088,
+    0x2497d08d, 0x2056cd3a, 0x2d15ebe3, 0x29d4f654,
+    0xc5a92679, 0xc1683bce, 0xcc2b1d17, 0xc8ea00a0,
+    0xd6ad50a5, 0xd26c4d12, 0xdf2f6bcb, 0xdbee767c,
+    0xe3a1cbc1, 0xe760d676, 0xea23f0af, 0xeee2ed18,
+    0xf0a5bd1d, 0xf464a0aa, 0xf9278673, 0xfde69bc4,
+    0x89b8fd09, 0x8d79e0be, 0x803ac667, 0x84fbdbd0,
+    0x9abc8bd5, 0x9e7d9662, 0x933eb0bb, 0x97ffad0c,
+    0xafb010b1, 0xab710d06, 0xa6322bdf, 0xa2f33668,
+    0xbcb4666d, 0xb8757bda, 0xb5365d03, 0xb1f740b4
+};
+
+#else
+
+static unsigned int s_CRC32[256];
 
 static void s_CRC32_Init(void)
 {
     size_t i;
+
     if (s_CRC32[255])
         return;
+
     for (i = 0;  i < 256;  i++) {
         unsigned int byteCRC = (unsigned int) i << 24;
         int j;
         for (j = 0;  j < 8;  j++) {
-            if (byteCRC & 0x80000000UL)
-                byteCRC = (byteCRC << 1) ^ CRC32_POLY;
-            else
-                byteCRC = (byteCRC << 1);
+            if (byteCRC & 0x80000000UL) {
+                byteCRC <<= 1;
+                byteCRC  ^= CRC32_POLY;
+            } else
+                byteCRC <<= 1;
         }
         s_CRC32[i] = byteCRC;
     }
 }
 
+#endif /*NCBI_USE_PRECOMPILED_CRC32_TABLES*/
+
 
 extern unsigned int CRC32_Update(unsigned int checksum,
                                  const void *ptr, size_t count)
 {
-    const unsigned char* str = (const unsigned char*) ptr;
     size_t j;
+    const char* str = (const char*) ptr;
 
+#ifndef NCBI_USE_PRECOMPILED_CRC32_TABLES
     s_CRC32_Init();
+#endif /*NCBI_USE_PRECOMPILED_CRC32_TABLES*/
+
     for (j = 0;  j < count;  j++) {
         size_t i = ((checksum >> 24) ^ *str++) & 0xFF;
         checksum <<= 8;
         checksum  ^= s_CRC32[i];
     }
+
     return checksum;
 }
 
@@ -761,125 +935,3 @@ extern int/*bool*/ UTIL_MatchesMask(const char* name, const char* mask)
 {
     return UTIL_MatchesMaskEx(name, mask, 1/*ignore case*/);
 }
-
-
-/*
- * ---------------------------------------------------------------------------
- * $Log: ncbi_util.c,v $
- * Revision 6.37  2006/07/26 14:38:07  lavr
- * UTIL_MatchesMaskEx():  rewrite to avoid use of NCBI_FAKE_WARNING
- *
- * Revision 6.36  2006/07/25 19:06:57  lavr
- * Mark fake WorkShop warning using NCBI_FAKE_WARNING -- by ivanov
- *
- * Revision 6.35  2006/06/15 02:45:13  lavr
- * GetUsername, GetVMPageSize (got CORE prefix), CRC32 moved here
- *
- * Revision 6.34  2006/04/14 20:09:28  lavr
- * +UTIL_MatchesMaskEx()
- *
- * Revision 6.33  2005/07/11 18:09:33  lavr
- * +UTIL_MatchesMask()
- *
- * Revision 6.32  2005/04/20 18:15:42  lavr
- * -"ncbi_config.h"
- *
- * Revision 6.31  2003/11/14 13:04:38  lavr
- * Little changes in comments [no code changes]
- *
- * Revision 6.30  2003/11/13 19:53:41  rsmith
- * Took out metrowerks specific #ifdef's (COMP_METRO). Not needed anymore.
- *
- * Revision 6.29  2003/09/02 21:05:14  lavr
- * Proper indentation of compilation conditionals
- *
- * Revision 6.28  2003/05/05 20:19:13  lavr
- * LOG_ComposeMessage() to check raw_size instead of raw_data ptr
- *
- * Revision 6.27  2003/05/05 11:41:09  rsmith
- * added defines and declarations to allow cross compilation Mac->Win32
- * using Metrowerks Codewarrior.
- *
- * Revision 6.26  2003/01/17 15:55:13  lavr
- * Fix errno reporting (comma was missing if errno == 0)
- *
- * Revision 6.25  2003/01/17 01:23:07  lavr
- * Always print full message for TRACE log in Debug mode
- *
- * Revision 6.24  2002/12/04 21:00:53  lavr
- * -CORE_LOG[F]_SYS_ERRNO()
- *
- * Revision 6.23  2002/12/04 19:51:12  lavr
- * No change
- *
- * Revision 6.22  2002/10/11 19:52:10  lavr
- * Log moved to end
- *
- * Revision 6.21  2002/06/18 17:07:44  lavr
- * Employ _strdate() & _strtime() if compiled by MSVC
- *
- * Revision 6.20  2002/05/07 18:22:10  lavr
- * Use fLOG_None in LOG_ComposeMessage()
- *
- * Revision 6.19  2002/02/11 20:36:44  lavr
- * Use "ncbi_config.h"
- *
- * Revision 6.18  2002/02/05 22:02:17  lavr
- * Minor tweak
- *
- * Revision 6.17  2002/01/28 20:22:39  lavr
- * Get rid of GCC warning about "'%D' yields only 2 last digits of year"
- *
- * Revision 6.16  2001/08/28 17:49:45  thiessen
- * oops, sorry - incorrect fix; reverted
- *
- * Revision 6.15  2001/08/28 17:21:22  thiessen
- * need ncbiconf.h for NCBI_CXX_TOOLKIT
- *
- * Revision 6.14  2001/08/09 16:25:06  lavr
- * Remove last (unneeded) parameter from LOG_Reset()
- * Added: fLOG_OmitNoteLevel format flag handling
- *
- * Revision 6.13  2001/07/30 14:41:37  lavr
- * Added: CORE_SetLOGFormatFlags()
- *
- * Revision 6.12  2001/07/26 15:13:02  lavr
- * Always do stream flush after message output (previously was in DEBUG only)
- *
- * Revision 6.11  2001/07/25 20:27:23  lavr
- * Included header files rearranged
- *
- * Revision 6.10  2001/07/25 19:12:57  lavr
- * Added date/time stamp for message logging
- *
- * Revision 6.9  2001/04/24 21:24:59  lavr
- * Make log flush in DEBUG mode
- *
- * Revision 6.8  2001/01/23 23:20:14  lavr
- * Comments added to some "boolean" 1s and 0s
- *
- * Revision 6.7  2001/01/12 23:50:38  lavr
- * "a+" -> "a" as a mode in fopen() for a logfile
- *
- * Revision 6.6  2000/08/28 20:05:51  vakatov
- * CORE_SetLOGFILE() -- typo fixed
- *
- * Revision 6.5  2000/06/23 19:34:45  vakatov
- * Added means to log binary data
- *
- * Revision 6.4  2000/05/30 23:23:26  vakatov
- * + CORE_SetLOGFILE_NAME()
- *
- * Revision 6.3  2000/03/31 17:19:11  kans
- * added continue statement to for loop to suppress missing body warning
- *
- * Revision 6.2  2000/03/24 23:12:09  vakatov
- * Starting the development quasi-branch to implement CONN API.
- * All development is performed in the NCBI C++ tree only, while
- * the NCBI C tree still contains "frozen" (see the last revision) code.
- *
- * Revision 6.1  2000/02/23 22:36:17  vakatov
- * Initial revision
- *
- * ===========================================================================
- */
