@@ -1,4 +1,4 @@
-/*  $Id: test_ncbi_connutil_hit.c,v 6.12 2002/08/12 15:10:43 lavr Exp $
+/*  $Id: test_ncbi_connutil_hit.c,v 6.14 2006/04/20 14:01:43 lavr Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -54,6 +54,8 @@ int main(int argc, char** argv)
     EIO_Status status;
     char buffer[10000];
 
+    CORE_SetLOGFormatFlags(fLOG_None          | fLOG_Level   |
+                           fLOG_OmitNoteLevel | fLOG_DateTime);
     CORE_SetLOGFILE(stderr, 0/*false*/);
 
     fprintf(stderr, "Running...\n"
@@ -160,6 +162,7 @@ int main(int argc, char** argv)
     /* Success:  close the socket, cleanup, and exit */
     SOCK_Close(sock);
     CORE_SetLOG(0);
+
     return 0;
 }
 
@@ -167,6 +170,12 @@ int main(int argc, char** argv)
 /*
  * ---------------------------------------------------------------------------
  * $Log: test_ncbi_connutil_hit.c,v $
+ * Revision 6.14  2006/04/20 14:01:43  lavr
+ * Cleanup to demonstrate no leaks
+ *
+ * Revision 6.13  2005/07/22 16:09:43  lavr
+ * Add severity and timestamp to log message format
+ *
  * Revision 6.12  2002/08/12 15:10:43  lavr
  * Use persistent SOCK_Write()
  *
