@@ -1,4 +1,4 @@
-/*  $Id: test_ncbi_socket.c,v 6.23 2006/03/31 19:58:22 lavr Exp $
+/*  $Id: test_ncbi_socket.c,v 6.25 2007/07/06 16:55:28 kazimird Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -683,6 +683,7 @@ static void TEST__server(unsigned short port)
  */
 static char TEST_LockUserData[] = "TEST_LockUserData";
 
+
 #if defined(__cplusplus)
 extern "C" {
     static int/*bool*/ TEST_LockHandler(void* user_data, EMT_Lock how);
@@ -704,6 +705,12 @@ static int/*bool*/ TEST_LockHandler(void* user_data, EMT_Lock how)
     case eMT_Unlock:
         what_str = "eMT_Unlock";
         break;
+    case eMT_TryLock:
+        what_str = "eMT_TryLock";
+        break;
+    case eMT_TryLockRead:
+        what_str = "eMT_TryLockRead";
+        break;
     }
     fprintf(log_fp, "TEST_LockHandler(\"%s\", %s)\n",
             user_data ? (char*)user_data : "<NULL>", what_str);
@@ -718,8 +725,6 @@ static void TEST_LockCleanup(void* user_data)
 }
 
 
-static int/*bool*/ TEST_gethostbyaddr(unsigned int host);
-
 static const char* s_ntoa(unsigned int host)
 {
     static char buf[256];
@@ -730,6 +735,7 @@ static const char* s_ntoa(unsigned int host)
     return buf;
 }
 
+
 static int/*bool*/ TEST_gethostbyname(const char* name)
 {
     char         buf[256];
@@ -935,85 +941,3 @@ extern int main(int argc, char** argv)
     CORE_SetLOCK(0);
     return 1;
 }
-
-
-/*
- * ---------------------------------------------------------------------------
- * $Log: test_ncbi_socket.c,v $
- * Revision 6.23  2006/03/31 19:58:22  lavr
- * Added more data logging
- *
- * Revision 6.22  2006/01/27 17:12:15  lavr
- * Replace obsolete call names with current ones
- *
- * Revision 6.21  2003/05/21 17:46:51  lavr
- * Fix MSVC-related problems with SOCK_Shutdown()
- *
- * Revision 6.20  2003/05/14 03:58:43  lavr
- * Match changes in respective APIs of the tests
- *
- * Revision 6.19  2002/12/04 16:58:13  lavr
- * No changes
- *
- * Revision 6.18  2002/11/01 20:17:39  lavr
- * Change hostname buffers to hold up to 256 chars
- *
- * Revision 6.17  2002/10/11 19:58:23  lavr
- * Remove asserts() (replace with tests) for SOCK_gethostbyaddr({0|0xFFFFFFFF})
- *
- * Revision 6.16  2002/08/12 15:10:43  lavr
- * Use persistent SOCK_Write()
- *
- * Revision 6.15  2002/08/07 16:38:08  lavr
- * EIO_ReadMethod enums changed accordingly; log moved to end
- *
- * Revision 6.14  2002/03/22 19:47:48  lavr
- * Test_assert.h made last among the include files
- *
- * Revision 6.13  2002/02/11 20:36:45  lavr
- * Use "ncbi_config.h"
- *
- * Revision 6.12  2002/01/16 21:23:15  vakatov
- * Utilize header "test_assert.h" to switch on ASSERTs in the Release mode too
- *
- * Revision 6.11  2001/07/11 00:44:33  vakatov
- * Added TEST_gethostby***() -- tests for SOCK_gethostby{addr,name}()
- *
- * Revision 6.10  2001/05/21 15:11:13  ivanov
- * Added test for automatic read on write data from the socket
- * (stall protection).
- *
- * Revision 6.9  2001/01/26 23:55:10  vakatov
- * [NCBI_OS_MAC]  Do not do server write shutdown for MAC client
- *
- * Revision 6.8  2000/11/15 18:51:44  vakatov
- * Add tests for SOCK_Shutdown() and SOCK_Status().
- * Use SOCK_Status() instead of SOCK_Eof().
- *
- * Revision 6.7  2000/06/23 19:39:22  vakatov
- * Test the logging of socket I/O (incl. binary data)
- *
- * Revision 6.6  2000/03/24 23:12:13  vakatov
- * Starting the development quasi-branch to implement CONN API.
- * All development is performed in the NCBI C++ tree only, while
- * the NCBI C tree still contains "frozen" (see the last revision) code.
- *
- * Revision 6.5  2000/02/24 23:09:42  vakatov
- * Use C++ Toolkit specific wrapper "test_ncbi_socket_.c" for
- * "test_ncbi_socket.c"
- *
- * Revision 6.4  2000/02/23 22:34:37  vakatov
- * Can work both "standalone" and as a part of NCBI C++ or C toolkits
- *
- * Revision 6.3  1999/11/26 19:05:21  vakatov
- * Initialize "log_fp" in "main()"...
- *
- * Revision 6.2  1999/10/19 16:16:03  vakatov
- * Try the NCBI C and C++ headers only if NCBI_OS_{UNIX, MSWIN, MAC} is
- * not #define'd
- *
- * Revision 6.1  1999/10/18 15:40:21  vakatov
- * Initial revision
- *
- * ===========================================================================
- */
