@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   3/4/91
 *
-* $Revision: 6.37 $
+* $Revision: 6.40 $
 *
 * File Description: 
 *     portable file routines
@@ -43,6 +43,15 @@
 * 11-27-94 Ostell      moved includes to ncbiwin.h to avoid conflict MSC
 *
 * $Log: ncbifile.c,v $
+* Revision 6.40  2007/08/16 17:09:22  kans
+* moved DirExplore from sqnutils
+*
+* Revision 6.39  2006/07/13 17:10:36  bollin
+* use Uint4 instead of Uint2 for itemID values
+*
+* Revision 6.38  2005/04/20 20:14:31  lavr
+* +<assert.h>
+*
 * Revision 6.37  2004/07/21 18:08:30  kans
 * FileCacheSetup calls _setmode (_fileno (fp), _O_BINARY) if OS_MSWIN
 *
@@ -204,6 +213,7 @@ Removed disabled CD routines.
 #define THIS_FILE  _this_file
 
 #include <ncbilcl.h>
+#include <assert.h>
 
 #include "corepriv.h"
 #ifdef OS_MAC
@@ -1262,6 +1272,89 @@ NLM_EXTERN ValNodePtr LIBCALL Nlm_DirCatalog (Nlm_CharPtr pathname)
   return vnp;
 }
 
+/*****************************************************************************
+*
+*   general file recursion functio
+*
+*****************************************************************************/
+
+NLM_EXTERN Nlm_Int4 Nlm_DirExplore (
+  Nlm_CharPtr directory,
+  Nlm_CharPtr filter,
+  Nlm_CharPtr suffix,
+  Nlm_Boolean recurse,
+  Nlm_DirExpProc proc,
+  Nlm_VoidPtr userdata
+)
+
+{
+  Nlm_Int4     count = 0;
+  Nlm_Char     file [FILENAME_MAX], path [PATH_MAX];
+  Nlm_CharPtr  ptr, str;
+  ValNodePtr   head, vnp;
+
+  if (proc == NULL) return 0;
+  if (Nlm_StringHasNoText (directory) || Nlm_StringHasNoText (suffix)) return 0;
+
+  /* get list of all files in source directory */
+
+  head = Nlm_DirCatalog (directory);
+
+  for (vnp = head; vnp != NULL; vnp = vnp->next) {
+    if (vnp->choice == 0) {
+      str = (Nlm_CharPtr) vnp->data.ptrvalue;
+      if (! Nlm_StringHasNoText (str)) {
+
+        /* check filename for indicated suffix */
+
+        ptr = Nlm_StringStr (str, suffix);
+        if (ptr != NULL) {
+
+          /* make sure detected suffix is really at end of filename */
+
+          if (Nlm_StringCmp (ptr, suffix) == 0) {
+            *ptr = '\0';
+          } else {
+            ptr = NULL;
+          }
+        }
+
+        if (Nlm_StringHasNoText (suffix) || ptr != NULL) {
+
+          Nlm_StringNCpy_0 (path, directory, sizeof (path));
+          sprintf (file, "%s%s", str, suffix);
+          Nlm_FileBuildPath (path, NULL, file);
+
+          /* check full path/file name for desired filter */
+
+          if (Nlm_StringHasNoText (filter) || Nlm_StringStr (path, filter) != NULL) {
+
+            /* process file that satisfies optional filter and suffix constraints */
+
+            proc (path, userdata);
+            count++;
+          }
+        }
+      }
+    } else if (vnp->choice == 1 && recurse) {
+
+      /* recurse into subdirectory */
+
+      Nlm_StringNCpy_0 (path, directory, sizeof (path));
+      str = (Nlm_CharPtr) vnp->data.ptrvalue;
+      Nlm_FileBuildPath (path, str, NULL);
+
+      count += Nlm_DirExplore (path, filter, suffix, recurse, proc, userdata);
+    }
+  }
+
+  /* clean up file list */
+
+  ValNodeFreeData (head);
+
+  return count;
+}
+
 /*****************************************************************************
 *
 *   TmpNam()
@@ -1486,7 +1579,7 @@ NLM_EXTERN Nlm_CharPtr Nlm_FileCacheGetString (
 
 {
   Nlm_Char     ch;
-  Nlm_Int2     count;
+  Nlm_Uint2     count;
   Nlm_CharPtr  ptr;
 
   if (fcp == NULL || fcp->fp == NULL || str == NULL || size < 1) return NULL;
