@@ -29,7 +29,7 @@
 *   
 * Version Creation Date: 1/1/91
 *
-* $Revision: 6.2 $
+* $Revision: 6.4 $
 *
 * File Description:  Object manager for module NCBI-Medline
 *
@@ -41,6 +41,12 @@
 *
 *
 * $Log: objmedli.c,v $
+* Revision 6.4  1998/08/26 17:43:17  kans
+* fixed -v -fd warnings in label functions
+*
+* Revision 6.3  1998/08/24 18:28:06  kans
+* removed solaris -v -fd warnings
+*
 * Revision 6.2  1997/10/28 16:09:11  grisha
 * fixed bug in function MedlineEntryAsnRead() to load status
 *
@@ -124,6 +130,7 @@ static Pointer LIBCALLBACK MedlineEntryAsnReadFunc (AsnIoPtr aip, AsnTypePtr atp
 
 static Int2 LIBCALLBACK MedlineEntryLabelFunc ( Pointer data, CharPtr buffer, Int2 buflen, Uint1 content)
 {
+	Int2 rsult = 0;
 
 	switch (content)
 	{
@@ -132,9 +139,9 @@ static Int2 LIBCALLBACK MedlineEntryLabelFunc ( Pointer data, CharPtr buffer, In
 		case OM_LABEL_SUMMARY:
 		case OM_LABEL_TYPE:
 		default:
-			return LabelCopy(buffer, typename, buflen);
+			rsult = LabelCopy(buffer, typename, buflen);
 	}
-	return 0;
+	return rsult;
 }
 
 /*****************************************************************************
@@ -148,7 +155,7 @@ static Int2 LIBCALLBACK MedlineEntryLabelFunc ( Pointer data, CharPtr buffer, In
 *   MedlineAsnLoad()
 *
 *****************************************************************************/
-NLM_EXTERN Boolean LIBCALL MedlineAsnLoad ()
+NLM_EXTERN Boolean LIBCALL MedlineAsnLoad (void)
 {
     if (loaded)
         return TRUE;
@@ -182,7 +189,7 @@ NLM_EXTERN Boolean LIBCALL MedlineAsnLoad ()
 *   MedlineMeshNew()
 *
 *****************************************************************************/
-NLM_EXTERN MedlineMeshPtr LIBCALL MedlineMeshNew ()
+NLM_EXTERN MedlineMeshPtr LIBCALL MedlineMeshNew (void)
 {
 	MedlineMeshPtr mmp;
 
