@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   7/1/91
 *
-* $Revision: 6.8 $
+* $Revision: 6.12 $
 *
 * File Description: 
 *       Vibrant button functions
@@ -41,6 +41,18 @@
 *
 *
 * $Log: vibbutns.c,v $
+* Revision 6.12  2007/05/01 22:01:30  kans
+* changes in preparation for supporing Quartz on Macintosh
+*
+* Revision 6.11  2006/09/14 19:18:28  ivanov
+* Rollback last changes. All missed defines added to corelib/ncbiwin.h.
+*
+* Revision 6.10  2006/09/14 18:05:45  ivanov
+* Fixed compilation errors on MS Windows
+*
+* Revision 6.9  2006/09/14 14:45:38  kans
+* changes for 64-bit Windows (GC) plus a few CodeWarrior complaints (JK)
+*
 * Revision 6.8  2004/01/20 23:34:38  sinyakov
 * [WIN_MSWIN]: fixed to display '&' character in button text
 *
@@ -137,15 +149,15 @@
 #include <vibincld.h>
 
 #ifdef WIN_MAC
-#define Nlm_ControlTool ControlHandle
+#  define Nlm_ControlTool ControlHandle
 #endif
 
 #ifdef WIN_MSWIN
-#define Nlm_ControlTool HWND
+#  define Nlm_ControlTool HWND
 #endif
 
 #ifdef WIN_MOTIF
-#define Nlm_ControlTool Widget
+#  define Nlm_ControlTool Widget
 #endif
 
 typedef  struct  Nlm_buttondata {
@@ -586,7 +598,7 @@ static void Nlm_EnableButton (Nlm_GraphiC b, Nlm_Boolean setFlag, Nlm_Boolean sa
     tempPort = Nlm_SavePortIfNeeded (b, savePort);
     c = Nlm_GetButtonHandle ((Nlm_ButtoN) b);
 #ifdef WIN_MAC
-    HiliteControl (c, 0);
+    ActivateControl (c);
 #endif
 #ifdef WIN_MSWIN
     EnableWindow (c, TRUE);
@@ -610,7 +622,7 @@ static void Nlm_DisableButton (Nlm_GraphiC b, Nlm_Boolean setFlag, Nlm_Boolean s
   tempPort = Nlm_SavePortIfNeeded (b, savePort);
   c = Nlm_GetButtonHandle ((Nlm_ButtoN) b);
 #ifdef WIN_MAC
-  HiliteControl (c, 255);
+  DeactivateControl (c);
 #endif
 #ifdef WIN_MSWIN
   EnableWindow (c, FALSE);
@@ -765,7 +777,7 @@ static void Nlm_SetButtonStatus (Nlm_GraphiC b, Nlm_Int2 item,
     val = 0;
   }
 #ifdef WIN_MAC
-  SetControlValue (c, val);
+  SetControl32BitValue (c, val);
 #endif
 #ifdef WIN_MSWIN
   Button_SetCheck (c, val);
@@ -783,7 +795,7 @@ static Nlm_Boolean Nlm_GetButtonStatus (Nlm_GraphiC b, Nlm_Int2 item)
 
   c = Nlm_GetButtonHandle ((Nlm_ButtoN) b);
 #ifdef WIN_MAC
-  return (GetControlValue (c) != 0);
+  return (GetControl32BitValue (c) != 0);
 #endif
 #ifdef WIN_MSWIN
   return (Nlm_Boolean) (Button_GetCheck (c) != 0);
@@ -924,7 +936,7 @@ static void MyCls_OnChar (HWND hwnd, UINT ch, int cRepeat)
   }
 }
 
-LRESULT CALLBACK EXPORT ButtonProc (HWND hwnd, UINT message,
+static LRESULT CALLBACK EXPORT ButtonProc (HWND hwnd, UINT message,
                                     WPARAM wParam, LPARAM lParam)
 
 {
@@ -1075,6 +1087,7 @@ static void Nlm_NewButton (Nlm_ButtoN b, Nlm_CharPtr title,
 #ifdef WIN_MAC
   Nlm_Int2         procID;
   Nlm_RectTool     rtool;
+  CFStringRef      cfTitle;
 #endif
 #ifdef WIN_MSWIN
   Nlm_Uint4        style;
@@ -1100,33 +1113,31 @@ static void Nlm_NewButton (Nlm_ButtoN b, Nlm_CharPtr title,
   dflt = (Nlm_Boolean) (type == DEFAULT_STYLE);
 
 #ifdef WIN_MAC
-  Nlm_CtoPstr (temp);
+  cfTitle = CFStringCreateWithCString(NULL, temp, kCFStringEncodingMacRoman);
   switch (type) {
-    case PUSH_STYLE:
-      procID = 0;
-      r.right -= shrinkX;
-      r.bottom -= shrinkY;
-      break;
     case DEFAULT_STYLE:
-      procID = 0;
       if (! Nlm_HasAquaMenuLayout ()) {
         border = 5;
       }
+      /* fall through */
+    case PUSH_STYLE:
       r.right -= shrinkX;
       r.bottom -= shrinkY;
+      Nlm_RecTToRectTool (&r, &rtool);
+      CreatePushButtonControl(wptr, &rtool, cfTitle, &c);
       break;
     case CHECK_STYLE:
-      procID = 1;
+      Nlm_RecTToRectTool (&r, &rtool);
+      CreateCheckBoxControl(wptr, &rtool, cfTitle, 0, FALSE, &c);
       break;
     case RADIO_STYLE:
-      procID = 2;
+      Nlm_RecTToRectTool (&r, &rtool);
+      CreateRadioButtonControl(wptr, &rtool, cfTitle, 0, FALSE, &c);
       break;
     default:
       procID = 0;
       break;
   }
-  Nlm_RecTToRectTool (&r, &rtool);
-  c = NewControl (wptr, &rtool, (StringPtr) temp, FALSE, 0, 0, 1, procID, 0);
   Nlm_LoadButtonData (b, c, border, offset, shrinkX, shrinkY, dflt);
 #endif
 
@@ -1160,11 +1171,11 @@ static void Nlm_NewButton (Nlm_ButtoN b, Nlm_CharPtr title,
     lpfnNewButtonProc = (WNDPROC) MakeProcInstance ((FARPROC) ButtonProc, Nlm_currentHInst);
   }
   if (lpfnOldButtonProc == NULL) {
-    lpfnOldButtonProc = (WNDPROC) GetWindowLong (c, GWL_WNDPROC);
-  } else if (lpfnOldButtonProc != (WNDPROC) GetWindowLong (c, GWL_WNDPROC)) {
+    lpfnOldButtonProc = (WNDPROC) GetWindowLongPtr (c, GWLP_WNDPROC);
+  } else if (lpfnOldButtonProc != (WNDPROC) GetWindowLongPtr (c, GWLP_WNDPROC)) {
     Nlm_Message (MSG_ERROR, "ButtonProc subclass error");
   }
-  SetWindowLong (c, GWL_WNDPROC, (LONG) lpfnNewButtonProc);
+  SetWindowLongPtr (c, GWLP_WNDPROC, (LONG) lpfnNewButtonProc);
   fntptr = (Nlm_FntPtr) Nlm_HandLock (Nlm_systemFont);
   SetWindowFont(c, fntptr->handle, FALSE);
   Nlm_HandUnlock(Nlm_systemFont);
