@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   7/1/91
 *
-* $Revision: 6.29 $
+* $Revision: 6.32 $
 *
 * File Description: 
 *       Vibrant procedure definitions
@@ -37,6 +37,15 @@
 * Modifications:  
 * --------------------------------------------------------------------------
 * $Log: vibprocs.h,v $
+* Revision 6.32  2007/05/01 22:01:30  kans
+* changes in preparation for supporing Quartz on Macintosh
+*
+* Revision 6.31  2006/09/27 18:30:00  kans
+* support for Int4 scroll bars for switching between text and doc views in Sequin (CB)
+*
+* Revision 6.30  2005/09/23 20:30:10  kans
+* added NLM_BACK for backspace key, which is separate from delete, even though the label may be the same on a keyboard
+*
 * Revision 6.29  2005/01/24 15:01:25  kans
 * prototype UsePrimaryMonitor
 *
@@ -558,6 +567,10 @@ void      Nlm_SetTextSelect PROTO((Nlm_TexT t, Nlm_TxtActnProc slct, Nlm_TxtActn
 
 Nlm_TexT  Nlm_CurrentText PROTO((void));
 
+extern void Nlm_SetScrollTextOffset4 (Nlm_GraphiC t, Nlm_Int4 horiz,
+                                     Nlm_Int4 vert, Nlm_Boolean savePort);
+extern void Nlm_GetScrollTextOffset4 (Nlm_GraphiC t, Nlm_Int4Ptr horiz, Nlm_Int4Ptr vert);
+
 /* return TRUE and selection range [begin,end] -- if there is a selection
  */
 Nlm_Boolean Nlm_TextSelectionRange PROTO((Nlm_TexT t, Nlm_Int4Ptr begin, Nlm_Int4Ptr end));
@@ -984,7 +997,7 @@ extern Nlm_Boolean Nlm_Execv PROTO((const Nlm_Char* path, Nlm_Char *const * argv
 
 /***  OBJECT ALIGNMENT PROCEDURE FOR USE WITHOUT GROUP REPOSITIONING  ***/
 
-#if defined(WIN_MAC)
+#if defined(WIN_MAC)  ||  defined(WIN_MAC_QUARTZ)
 #define Nlm_HANDLE Nlm_Handle
 #elif defined(WIN_MOTIF)
 #define Nlm_HANDLE Nlm_Handle
@@ -1015,6 +1028,7 @@ void CDECL   Nlm_AlignObjects VPROTO((int align, ...));
 
 #define NLM_DEL    127
 #define NLM_INS      5
+#define NLM_BACK     8
 #define NLM_ESC     27
 
 #define NLM_LEFT    28
